package com.hungraim.ltc.account.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hungraim.ltc.account.config.AuditingRoleConfig;
import com.hungraim.ltc.account.dao.*;
import com.hungraim.ltc.account.service.AccountService;
import com.hungraim.ltc.account.service.BaseAccountService;
import com.hungraim.ltc.api.AttachFeignService;
import com.hungraim.ltc.pojo.entity.SysConfig.ChCfgCareService;
import com.hungraim.ltc.pojo.entity.SysConfig.ChCfgCostStandard;
import com.hungraim.ltc.pojo.entity.SysConfig.ChFndAttach;
import com.hungraim.ltc.pojo.entity.account.*;
import com.hungraim.ltc.pojo.entity.governance.ChSrvPlan;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvTaskDetail;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvTaskDetailAuditLog;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvTaskFinishItem;
import com.hungraim.ltc.pojo.vo.account.*;
import com.hungraim.ltc.pojo.vo.disable.PlaceOtherVO;
import com.hungraim.ltc.util.*;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Service
@AllArgsConstructor
@Slf4j
public class AccountServiceImpl extends ServiceImpl<ChDisabAccountsMapper, ChDisabAccounts> implements AccountService {

    private final AttachFeignService attachFeignService;
    private final BaseAccountService baseAccountService;
    private final ChDisabAccountsMapper disabAccountsMapper;
    private final SrvTaskMapper srvTaskMapper;
    private final ChSrvTaskDetailMapper taskDetailMapper;
    private final ChSrvTaskFinishItemMapper taskFinishItemMapper;
    private final ChSrvProgramMapper programMapper;
    private final ChSrvProgramSetMapper programSetMapper;
    private final ChCfgCareServiceMapper chCfgCareServiceMapper;
    private final ChCfgCostStandardMapper chCfgCostStandardMapper;
    private final ChDisabAccountsDetailMapper chDisabAccountsDetailMapper;
    private final ChDisabFreeRdInfoMapper chDisabFreeRdInfoMapper;
    private final ChSrvPlanMapper chSrvPlanMapper;
    private final ChSrvTaskDetailAuditLogMapper chSrvTaskDetailAuditLogMapper;


    @Autowired
    private  AuditingRoleConfig auditingRoleConfig;

    private final ChSrvTaskDetailMapper chSrvTaskDetailMapper;
    private final ChSrvTaskFinishItemMapper chSrvTaskFinishItemMapper;;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result accountReportReject(ReportRejectReqVO reportRejectReqVO) {
        ChDisabDetailAccounts chDisabDetailAccounts = new ChDisabDetailAccounts();
        chDisabDetailAccounts.setLcd(new Date());
        chDisabDetailAccounts.setAccountsDetailId(reportRejectReqVO.getAccountsDetailId());
        chDisabDetailAccounts.setStatus((short)0);
        chDisabDetailAccounts.setDealType(0);
        int result = chDisabAccountsDetailMapper.updateById(chDisabDetailAccounts);
        if (result > 0) {
            ChDisabAccounts chDisabAccountDTO = disabAccountsMapper.selectById(reportRejectReqVO.getAccountsId());
            BigDecimal allCost = chDisabAccountDTO.getAllCost();
            BigDecimal overallCost = chDisabAccountDTO.getOverallCost();
            BigDecimal personalCost = chDisabAccountDTO.getPersonalCost();
            BigDecimal allCostSubtract = allCost.subtract(reportRejectReqVO.getAccountsAllCost());
            BigDecimal overallCostSubtract = overallCost.subtract(reportRejectReqVO.getAccountsOverallCost());
            BigDecimal personalCostSubtract = personalCost.subtract(reportRejectReqVO.getAccountsPersonalCost());
            chDisabAccountDTO.setAllCost(allCostSubtract);
            chDisabAccountDTO.setOverallCost(overallCostSubtract);
            chDisabAccountDTO.setPersonalCost(personalCostSubtract);
            chDisabAccountDTO.setLcd(new Date());
            disabAccountsMapper.updateById(chDisabAccountDTO);
            return Result.success();
        } else {
            return Result.failed("驳回失败");
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addOrUpdateAccount(AccountOperVo accountOperVo) {
        ChDisabAccounts chDisabAccounts = new ChDisabAccounts();
        BeanUtils.copyProperties(accountOperVo, chDisabAccounts);
        chDisabAccounts.setChooseSettle((short) 0); // 是否结算(0未结算，1已结算)
        chDisabAccounts.setReviewRemark(accountOperVo.getReviewRemark());
        Short accountsStatus = accountOperVo.getAccountsStatus();
        String accountsId = String.valueOf(accountOperVo.getAccountsId());
        String modeName = accountOperVo.getModeName();
        // 驳回
        if(accountsStatus == 0) {
            chDisabAccountsDetailMapper.updateByAccountsId(accountsId,0);
            chDisabAccounts.setAllCost(BigDecimal.valueOf(0));
            chDisabAccounts.setOverallCost(BigDecimal.valueOf(0));
            chDisabAccounts.setPersonalCost(BigDecimal.valueOf(0));
            chDisabAccounts.setReissueAmountTotal(BigDecimal.valueOf(0));
            chDisabAccounts.setDeductionAmountTotal(BigDecimal.valueOf(0));
            HashMap map = new HashMap();
            map.put("ACCOUNTS_ID",accountsId);
            // 删除对应扣减历史数据
            chDisabFreeRdInfoMapper.deleteByMap(map);

        }
        // 已结算
        if(accountsStatus == 4) {
            ChDisabAccounts chDisabAccountDTO = disabAccountsMapper.selectById(accountOperVo.getAccountsId());
            chDisabAccountDTO.setTotalAllocatedExpenses(chDisabAccountDTO.getTotalAllocatedExpenses());
            chDisabAccountDTO.setChooseSettle((short) 1); // 是否结算(0未结算，1已结算)
            disabAccountsMapper.updateById(chDisabAccountDTO);
        }
        chDisabAccounts.setLcd(new Date());
        chDisabAccounts.setLcu(accountOperVo.getUserId());
        this.saveOrUpdate(chDisabAccounts);
        }

    @Override
    public Page<SrvOrganListResp> searchSrvOrganList(SrvOrganListReq srvOrganListReq) {
        List<Long> organIds = baseAccountService.getOrganIds(srvOrganListReq.getOrganId(), srvOrganListReq.getIsInvolveChild());
        return disabAccountsMapper.searchSrvOrganList(new Page<>(srvOrganListReq.getPageNum(), srvOrganListReq.getPageSize()), organIds, srvOrganListReq);
    }

    @Override
    public Page<SrvOrganAccountListResp> searchSrvOrganAccountList(SrvOrganListReq srvOrganListReq) {
        List<Long> organIds = baseAccountService.getOrganIds(srvOrganListReq.getOrganId(), srvOrganListReq.getIsInvolveChild());
        Page<SrvOrganAccountListResp> srvOrganAccountListRespPage = disabAccountsMapper.searchSrvOrganAccountList(new Page<>(srvOrganListReq.getPageNum(), srvOrganListReq.getPageSize()), organIds, srvOrganListReq);
        List<SrvOrganAccountListResp> records = srvOrganAccountListRespPage.getRecords();
        for (SrvOrganAccountListResp record : records) {
            Short accountsStatus = record.getAccountsStatus();
            if(accountsStatus == 4){
                Long accountsId = record.getAccountsId();
                // 1 补发 2 扣减
                List<String> list = chDisabFreeRdInfoMapper.selectByAccountsId(accountsId, 1);
                // 补发
                record.setReissueReason(list);
                List<String> list1 = chDisabFreeRdInfoMapper.selectByAccountsId(accountsId, 2);
                // 扣减
                record.setDeductionReason(list1);
                Date accountsTime = record.getAccountsTime();
                String accountsTimeStr = DateUtils.dateToStr(accountsTime);
                Date firstDayOfMonth = DateUtils.getFirstDayOfMonth(DateUtils.strToDate(accountsTimeStr));
                record.setSvrStartTime(DateUtils.dateToStr(firstDayOfMonth));
                Date date = DateUtils.getlastDayOfMonth(DateUtils.strToDate(accountsTimeStr));
                record.setSvrEndTime(DateUtils.dateToStr(date));

            }

        }
        return srvOrganAccountListRespPage;
    }




    @Override
    public AccountOperVo getSrvOrganDetail(Long accountsId) {
        AccountOperVo accountOperVo = disabAccountsMapper.getDetailById(accountsId);
        try {
            Result<List<ChFndAttach>> result = attachFeignService.selectAttachs(accountOperVo.getAccountsId(), AttachType.SRV_ORGAN_ACCOUNT.getUploadType());
            if (ResultCode.SUCCESS.getCode().equals(result.getCode())) {
                accountOperVo.setFileDataList(result.getData());
            } else {
                log.error("获取机构结算资料失败：{}", result.getMsg());
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取机构结算资料失败：{}", e.getMessage());
        }
        return accountOperVo;
    }

    @Override
    public ExpenseVo expenseSettlement(ExpenseSettlementReq req) throws ParseException {
        Page<ExpenseSettlementInfoBo> page = new Page<>(req.getPageNum(), req.getPageSize());
        //计算当前月最后一天
        DateFormat df = new SimpleDateFormat("yyyy-MM");
        Date parse = df.parse(req.getAccountsTime());
        Date lastDayOfMonth = getLastDayOfMonth(parse);
        DateFormat df2 = new SimpleDateFormat("yyyy-MM-dd");
        req.setAccountsLastDayTime(df2.format(lastDayOfMonth));
        Page<ExpenseSettlementInfoBo> expenseSettlementInfo = srvTaskMapper.getExpenseSettlementInfo(page, req);
        List<ExpenseSettlementVo> vos = new ArrayList<>();
        if (expenseSettlementInfo != null && expenseSettlementInfo.getRecords() != null) {
            for (ExpenseSettlementInfoBo bo : expenseSettlementInfo.getRecords()) {
                ExpenseSettlementVo vo = new ExpenseSettlementVo();
                BeanUtil.copyProperties(bo, vo);
                //护理等级
                vo.setNursingGrade(CfgSrvLevel.getMsg(bo.getSrvLevelId()));
                //服务开始时间
                if (bo.getSrvStartTime() != null) {
                    if (bo.getSrvStartTime().getTime() > parse.getTime()) {
                        vo.setSrvStartTime(bo.getSrvStartTime());
                    } else {
                        vo.setSrvStartTime(parse);
                    }
                }
                //服务结束时间
                if (bo.getSrvEndTime() != null) {
                    if (bo.getSrvEndTime().getTime() > lastDayOfMonth.getTime()) {
                        vo.setSrvEndTime(lastDayOfMonth);
                    } else {
                        vo.setSrvEndTime(bo.getSrvEndTime());
                    }
                }
                //有效结算日
                int startDay = getDay(vo.getSrvStartTime());
                int endDay = getDay(vo.getSrvEndTime());
                vo.setEffectiveSettlementDate(endDay - startDay + 1);
                //基护费用标准
                Integer effectiveSettlementDate = vo.getEffectiveSettlementDate();//有效结算日
                ChCfgCostStandard costStandard = chCfgCostStandardMapper.selectOne(new LambdaQueryWrapper<ChCfgCostStandard>()
                        .eq(ChCfgCostStandard::getOrganId, bo.getOrganId())
                        .eq(ChCfgCostStandard::getSrvLevelId, bo.getSrvLevelId())
                        .eq(ChCfgCostStandard::getSrvModeId, bo.getSrvModeId()));//基护费用标准(可配置)
                vo.setCostStandard(costStandard + "元/人/天");
                //基护费用
                BigDecimal foundation = BigDecimal.valueOf(costStandard.getCost()).multiply(BigDecimal.valueOf(effectiveSettlementDate));
                vo.setFoundationProtectionCost(foundation.doubleValue());
                //基护完成率
                Integer srvTaskId = bo.getSrvTaskId();
                int taskDetailsCount = taskDetailMapper.selectCount(new LambdaQueryWrapper<ChSrvTaskDetail>()//计划次数
                        .eq(ChSrvTaskDetail::getSrvTaskId, srvTaskId)
                        .in(ChSrvTaskDetail::getIllegalType, 0, 1)
                        .ne(ChSrvTaskDetail::getStatus, 3));
                List<ChSrvTaskDetail> taskDetails = taskDetailMapper.selectList(new LambdaQueryWrapper<ChSrvTaskDetail>()
                        .eq(ChSrvTaskDetail::getSrvTaskId, srvTaskId)
                        .in(ChSrvTaskDetail::getIllegalType, 0, 1)
                        .eq(ChSrvTaskDetail::getStatus, 4));
                int taskDetailsFinishCount = 0;//完成次数
                if (taskDetails != null && taskDetails.size() > 0) {
                    taskDetailsFinishCount = taskDetails.size();
                }
                int monPlanningFrequency = bo.getPlanningFrequency();//每月计划次数
                if (bo.getPlanningFrequencyType() == 1) {
                    monPlanningFrequency = bo.getPlanningFrequency();
                } else if (bo.getPlanningFrequencyType() == 2) {
                    monPlanningFrequency = bo.getPlanningFrequency() * 30;
                } else if (bo.getPlanningFrequencyType() == 3) {
                    monPlanningFrequency = bo.getPlanningFrequency() * 4;
                }
                BigDecimal divide;
                if (taskDetailsCount == 0) {
                    divide = BigDecimal.valueOf(0);
                } else {
                    divide = BigDecimal.valueOf(taskDetailsFinishCount).divide(BigDecimal.valueOf(taskDetailsCount), 2, BigDecimal.ROUND_DOWN);
                }
                BigDecimal divide1 = BigDecimal.valueOf(bo.getItemTimes()).divide(BigDecimal.valueOf(monPlanningFrequency), 2, BigDecimal.ROUND_DOWN);
                if (divide.doubleValue() > 1) {
                    divide = BigDecimal.valueOf(1);
                }
                if (divide1.doubleValue() > 1) {
                    divide1 = BigDecimal.valueOf(1);
                }
                BigDecimal comp = divide.doubleValue() == 0 ? BigDecimal.valueOf(0) : divide.multiply(divide1).setScale(4, BigDecimal.ROUND_DOWN);//基护完成率
                vo.setCompletionRateFoundationProtection(comp.doubleValue() == 0 ? "0%" : comp.multiply(BigDecimal.valueOf(100)).doubleValue() + "%");
                //巡护基护完成考核
                vo.setPatrolAndBasicProtectionCompletedAssessment("-");
                BigDecimal patrolAndBasicProtectionCompletedAssessment = BigDecimal.valueOf(0.8);//巡护基护完成考核
                if (FndSrvMode.PATROL.getCode().equals(bo.getSrvModeId())) {//巡护
                    long serviceTime = 0;//服务时长/毫秒
                    int serviceCount = 0;//服务次数
                    if (taskDetails != null && taskDetails.size() > 0) {
                        for (ChSrvTaskDetail taskDetail : taskDetails) {
                            long l = taskDetail.getSrvEndTime().getTime() - taskDetail.getSrvStartTime().getTime();
                            serviceTime += l;
                            if (l > 30 * 60 * 1000) {
                                serviceCount++;
                            }
                        }
                    }
                    long serviceTimeM = 0;
                    if (serviceTime != 0) {
                        serviceTimeM = serviceTime / (1000 * 60 * 60);//服务时长/小时
                    }
                    boolean serviceCountK = false;//服务次数考核
                    boolean serviceTimeK = false;//服务时长考核
                    if (serviceCount >= 6) {
                        serviceCountK = true;
                    }
                    if (DisableLevel.ONE.getCode().equals(bo.getDisableLevelId())) {
                        if (serviceTimeM >= 8) {
                            serviceTimeK = true;
                        }
                    } else if (DisableLevel.TWO.getCode().equals(bo.getDisableLevelId())) {
                        if (serviceTimeM >= 10) {
                            serviceTimeK = true;
                        }
                    } else if (DisableLevel.THREE.getCode().equals(bo.getDisableLevelId())) {
                        if (serviceTimeM >= 12) {
                            serviceTimeK = true;
                        }
                    }
                    if (serviceCountK && serviceTimeK) {
                        patrolAndBasicProtectionCompletedAssessment = BigDecimal.valueOf(1);
                        vo.setPatrolAndBasicProtectionCompletedAssessment("100%");
                    } else if (serviceCountK || serviceTimeK) {
                        patrolAndBasicProtectionCompletedAssessment = BigDecimal.valueOf(0.9);
                        vo.setPatrolAndBasicProtectionCompletedAssessment("90%");
                    } else {
                        vo.setPatrolAndBasicProtectionCompletedAssessment("80%");
                    }
                }
                //基护应拨费用
                BigDecimal foundationProtectionCost = BigDecimal.valueOf(vo.getFoundationProtectionCost());//基护费用
                if (FndSrvMode.PATROL.getCode().equals(bo.getSrvModeId())) {//巡护
                    BigDecimal multiply = foundationProtectionCost.multiply(comp).multiply(patrolAndBasicProtectionCompletedAssessment).multiply(BigDecimal.valueOf(0.9)).setScale(2, BigDecimal.ROUND_DOWN);
                    vo.setExpensesPayableFoundationProtection(multiply.doubleValue());
                } else if (FndSrvMode.HOME_CARE.getCode().equals(bo.getSrvModeId())
                        || FndSrvMode.HOSPITAL_NURSING.getCode().equals(bo.getSrvModeId())) {//家护/院护
                    BigDecimal multiply = foundationProtectionCost.multiply(comp).multiply(BigDecimal.valueOf(0.9)).setScale(2, BigDecimal.ROUND_DOWN);
                    vo.setExpensesPayableFoundationProtection(multiply.doubleValue());
                }
                //照护费用标准
                int standardCareCost = 0;
                if (CfgSrvLevel.ONE.getCode().equals(bo.getSrvLevelId())) {//服务等级
                    standardCareCost = 25;
                    vo.setStandardCareCost("25元/人/天");
                } else if (CfgSrvLevel.TWO.getCode().equals(bo.getSrvLevelId())) {
                    standardCareCost = 35;
                    vo.setStandardCareCost("35元/人/天");
                } else if (CfgSrvLevel.THREE.getCode().equals(bo.getSrvLevelId())) {
                    standardCareCost = 45;
                    vo.setStandardCareCost("45元/人/天");
                }
                //照护费用
                List<ChSrvTaskDetail> taskDetails2 = taskDetailMapper.selectList(new LambdaQueryWrapper<ChSrvTaskDetail>()
                        .eq(ChSrvTaskDetail::getSrvTaskId, srvTaskId)
                        .in(ChSrvTaskDetail::getIllegalType, 0, 2)
                        .eq(ChSrvTaskDetail::getStatus, 4));
                BigDecimal careCosts = BigDecimal.valueOf(0);
                int taskDetailsFinishCount2 = 0;
                if (taskDetails2 != null && taskDetails2.size() > 0) {
                    taskDetailsFinishCount2 = taskDetails2.size();
                    careCosts = BigDecimal.valueOf(standardCareCost).multiply(BigDecimal.valueOf(taskDetails2.size()));
                }
                if (FndSrvMode.PATROL.getCode().equals(bo.getSrvModeId())) {//巡护
                    ChCfgCareService chCfgCareService = chCfgCareServiceMapper.selectOne(new LambdaQueryWrapper<ChCfgCareService>()
                            .eq(ChCfgCareService::getSrvlevelId, bo.getSrvLevelId()));//根据服务等级查询照护服务费用限额配置表
                    if (chCfgCareService != null) {
                        if (careCosts.intValue() > chCfgCareService.getMonthlyPaymentLimit()) {
                            careCosts = BigDecimal.valueOf(chCfgCareService.getMonthlyPaymentLimit());
                        }
                    }
                }
                vo.setCareCosts(careCosts.doubleValue());
                //照护完成率
                int taskDetailsCount2 = taskDetailMapper.selectCount(new LambdaQueryWrapper<ChSrvTaskDetail>()//计划次数
                        .eq(ChSrvTaskDetail::getSrvTaskId, srvTaskId)
                        .in(ChSrvTaskDetail::getIllegalType, 0, 2)
                        .ne(ChSrvTaskDetail::getStatus, 3));
                BigDecimal divide2;
                if (taskDetailsCount2 == 0) {
                    divide2 = BigDecimal.valueOf(0);
                } else {
                    divide2 = BigDecimal.valueOf(taskDetailsFinishCount2).divide(BigDecimal.valueOf(taskDetailsCount2), 2, BigDecimal.ROUND_DOWN);
                }
                BigDecimal divide3 = BigDecimal.valueOf(bo.getItemTimes()).divide(BigDecimal.valueOf(monPlanningFrequency), 2, BigDecimal.ROUND_DOWN);
                if (divide2.doubleValue() > 1) {
                    divide2 = BigDecimal.valueOf(1);
                }
                if (divide3.doubleValue() > 1) {
                    divide3 = BigDecimal.valueOf(1);
                }
                BigDecimal comp2 = divide2.doubleValue() == 0 ? BigDecimal.valueOf(0) : divide2.multiply(divide3).setScale(4, BigDecimal.ROUND_DOWN);//照护完成率
                vo.setCareCompletionRate(comp2.doubleValue() == 0 ? "0%" : comp2.multiply(BigDecimal.valueOf(100)).doubleValue() + "%");
                //巡护照护完成考核
                vo.setPatrolCareCompletionAssessment("-");
                BigDecimal patrolCareCompletionAssessment = BigDecimal.valueOf(0.8);//巡护照护完成考核
                if (FndSrvMode.PATROL.getCode().equals(bo.getSrvModeId())) {//巡护
                    long serviceTime = 0;//服务时长/毫秒
                    int serviceCount = 0;//服务次数
                    if (taskDetails2 != null && taskDetails2.size() > 0) {
                        for (ChSrvTaskDetail taskDetail : taskDetails2) {
                            long l = taskDetail.getSrvEndTime().getTime() - taskDetail.getSrvStartTime().getTime();
                            serviceTime += l;
                            if (l > 30 * 60 * 1000) {
                                serviceCount++;
                            }
                        }
                    }
                    long serviceTimeM = 0;
                    if (serviceTime != 0) {
                        serviceTimeM = serviceTime / (1000 * 60 * 60);//服务时长/小时
                    }
                    boolean serviceCountK = false;//服务次数考核
                    boolean serviceTimeK = false;//服务时长考核
                    if (serviceCount >= 6) {
                        serviceCountK = true;
                    }
                    if (DisableLevel.ONE.getCode().equals(bo.getDisableLevelId())) {
                        if (serviceTimeM >= 8) {
                            serviceTimeK = true;
                        }
                    } else if (DisableLevel.TWO.getCode().equals(bo.getDisableLevelId())) {
                        if (serviceTimeM >= 10) {
                            serviceTimeK = true;
                        }
                    } else if (DisableLevel.THREE.getCode().equals(bo.getDisableLevelId())) {
                        if (serviceTimeM >= 12) {
                            serviceTimeK = true;
                        }
                    }
                    if (serviceCountK && serviceTimeK) {
                        patrolCareCompletionAssessment = BigDecimal.valueOf(1);
                        vo.setPatrolCareCompletionAssessment("100%");
                    } else if (serviceCountK || serviceTimeK) {
                        patrolCareCompletionAssessment = BigDecimal.valueOf(0.9);
                        vo.setPatrolCareCompletionAssessment("90%");
                    } else {
                        vo.setPatrolCareCompletionAssessment("80%");
                    }
                }
                //照护应拨费用
                if (FndSrvMode.PATROL.getCode().equals(bo.getSrvModeId())) {//巡护
                    BigDecimal multiply = careCosts.multiply(comp2).multiply(patrolCareCompletionAssessment).multiply(BigDecimal.valueOf(0.9)).setScale(2, BigDecimal.ROUND_DOWN);
                    vo.setCarePayable(multiply.doubleValue());
                } else if (FndSrvMode.HOME_CARE.getCode().equals(bo.getSrvModeId())
                        || FndSrvMode.HOSPITAL_NURSING.getCode().equals(bo.getSrvModeId())) {//家护/院护
                    BigDecimal multiply = careCosts.multiply(comp2).multiply(BigDecimal.valueOf(0.9)).setScale(2, BigDecimal.ROUND_DOWN);
                    vo.setCarePayable(multiply.doubleValue());
                }
                //应拨付总金额
                vo.setTotalAmountDue(BigDecimal.valueOf(vo.getExpensesPayableFoundationProtection()).add(BigDecimal.valueOf(vo.getCarePayable())).doubleValue());
                vos.add(vo);
            }
            //合计
            ExpenseSettlementTotalVo totalVo = new ExpenseSettlementTotalVo();
            BigDecimal foundationProtectionCost = BigDecimal.valueOf(0);
            BigDecimal expensesPayableFoundationProtection = BigDecimal.valueOf(0);
            BigDecimal careCosts = BigDecimal.valueOf(0);
            BigDecimal carePayable = BigDecimal.valueOf(0);
            BigDecimal totalAmountDue = BigDecimal.valueOf(0);
            for (ExpenseSettlementVo vo : vos) {
                //基护费用
                foundationProtectionCost = foundationProtectionCost.add(BigDecimal.valueOf(vo.getFoundationProtectionCost()));
                //基护应拨费用
                expensesPayableFoundationProtection = expensesPayableFoundationProtection.add(BigDecimal.valueOf(vo.getExpensesPayableFoundationProtection()));
                //照护费用
                careCosts = careCosts.add(BigDecimal.valueOf(vo.getCareCosts()));
                //照护应拨费用
                carePayable = carePayable.add(BigDecimal.valueOf(vo.getCarePayable()));
                //应拨付总金额
                totalAmountDue = totalAmountDue.add(BigDecimal.valueOf(vo.getTotalAmountDue()));
            }
            totalVo.setFoundationProtectionCost(foundationProtectionCost.doubleValue());
            totalVo.setExpensesPayableFoundationProtection(expensesPayableFoundationProtection.doubleValue());
            totalVo.setCareCosts(careCosts.doubleValue());
            totalVo.setCarePayable(carePayable.doubleValue());
            totalVo.setTotalAmountDue(totalAmountDue.doubleValue());
            ExpenseVo expenseVo = new ExpenseVo();
            expenseVo.setTotal(expenseSettlementInfo.getTotal());
            expenseVo.setCurrent(expenseSettlementInfo.getCurrent());
            expenseVo.setSize(expenseSettlementInfo.getSize());
            expenseVo.setExpenseSettlements(vos);
            expenseVo.setTotalVo(totalVo);
            return expenseVo;
        }
        return new ExpenseVo();
    }

    //求当月最后一天
    private  Date getLastDayOfMonth(Date date) {
        Calendar cale = Calendar.getInstance();
        cale.setTime(date);
        cale.add(Calendar.MONTH, 1);
        cale.set(Calendar.DAY_OF_MONTH, 0);
        return cale.getTime();
    }




    //求年月日中的日
    private int getDay(Date date) {
        Calendar cale = Calendar.getInstance();
        cale.setTime(date);
        return cale.get(Calendar.DAY_OF_MONTH);
    }













    @Override
    public Workbook srvOrganAccountExport(SrvOrganListReq req) {
        List<AccountExportDetailListResp> srvOrganAccountExportList = chDisabAccountsDetailMapper.searchSrvOrganAccountDetailExport(req);
        ExportParams exportParams = new ExportParams();
        exportParams.setType(ExcelType.XSSF);    //对应xlsx
        Workbook workbook = ExcelExportUtil.exportExcel(exportParams, AccountExportDetailListResp.class, srvOrganAccountExportList);
        return workbook;
    }

    @Override
    public Workbook accountsDeclareExport(SrvOrganListReq req) {
        List<AccountDeclareExportResp> accountDeclareExportResps = chDisabAccountsDetailMapper.queryAccountsDeclareExport(req);
        ExportParams exportParams = new ExportParams();
        exportParams.setType(ExcelType.XSSF);    //对应xlsx
        Workbook workbook = ExcelExportUtil.exportExcel(exportParams, AccountDeclareExportResp.class, accountDeclareExportResps);
        return workbook;
    }



    @Override
    public Page<ChDisabDetailAccountsResp> queryOrganAccountsDetail(Integer pageNum, Integer pageSize,SrvOrganListReq srvOrganListReq) {
        Page<ChDisabDetailAccountsResp> chDisabDetailAccountsRespPage = chDisabAccountsDetailMapper.queryOrganAccountsDetail(new Page<>(pageNum, pageSize), srvOrganListReq);
 /*       List<ChDisabDetailAccountsResp> records = chDisabDetailAccountsRespPage.getRecords();
        for (ChDisabDetailAccountsResp record : records) {
            Integer status = record.getStatus();
            if(status == 1){
                BigDecimal reportableAmount = new BigDecimal(0);
                BigDecimal institutionsDeserve = new BigDecimal(0);
                BigDecimal personalDeserve = new BigDecimal(0);
                String accountsStartTime = record.getAccountsStartTime();
                if(StringUtils.isEmpty(accountsStartTime)){
                    accountsStartTime = record.getAccountsTime()+"-01";
                }
                Date date = DateUtils.strToDate(accountsStartTime);
                int daysOfMonth =  getDaysOfMonth(date);
                String modeName = record.getModeName();
                if ("上门".equals(modeName)) {
                    reportableAmount = new BigDecimal(1200);
                    institutionsDeserve = new BigDecimal(960);
                    personalDeserve = new BigDecimal(240);
                } else if ("养老".equals(modeName)) {
                    reportableAmount = new BigDecimal(50).multiply(new BigDecimal(daysOfMonth));
                    institutionsDeserve = reportableAmount.multiply(new BigDecimal("0.7"));
                    personalDeserve = reportableAmount.multiply(new BigDecimal("0.3"));
                } else if ("医疗".equals(modeName) && "1".equals(record.getLevelCode())) {
                    reportableAmount = new BigDecimal(80).multiply(new BigDecimal(daysOfMonth));
                    institutionsDeserve = reportableAmount.multiply(new BigDecimal("0.7"));
                    personalDeserve = reportableAmount.multiply(new BigDecimal("0.3"));
                    // 二级及以上医疗机构其限额为 100 元/人/天,由长期护理保险基金支付 70%，个人支付 30%;
                } else if ("医疗".equals(modeName) && (!"1".equals(record.getLevelCode()))) {
                    reportableAmount = new BigDecimal(100).multiply(new BigDecimal(daysOfMonth));
                    institutionsDeserve = reportableAmount.multiply(new BigDecimal("0.7"));
                    personalDeserve = reportableAmount.multiply(new BigDecimal("0.3"));
                }
                record.setReportableAmount(reportableAmount.toString());
                record.setInstitutionsDeserve(institutionsDeserve.toString());
                record.setPersonalDeserve(personalDeserve.toString());
            }

        }*/

        return chDisabDetailAccountsRespPage;
    }



    public static int getDaysOfMonth(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        return calendar.getActualMaximum(Calendar.DAY_OF_MONTH);
    }

    @Override
    public Page<NursingRecordsRespVO> queryNursingRecords(NursingReqVO nursingReqVO) {
        String modeName = nursingReqVO.getModeName();
        Page<NursingRecordsRespVO> nursingRecordsRespVOPage = null;
        if("上门".equals(modeName)){
            nursingRecordsRespVOPage = chDisabAccountsDetailMapper.queryNursingRecordsGoDoor(new Page<>(nursingReqVO.getPageNum(), nursingReqVO.getPageSize()), nursingReqVO);
            List<NursingRecordsRespVO> records = nursingRecordsRespVOPage.getRecords();
            for (NursingRecordsRespVO record : records) {
                String srvStartTime = record.getSrvStartTime();
                String srvEndTime = record.getSrvEndTime();
                if(StringUtils.isNotEmpty(srvStartTime) && StringUtils.isNotEmpty(srvEndTime)) {
                    String time = dateDiff(srvStartTime, srvEndTime, "yyyy-MM-dd HH:mm:ss", "min");
                    record.setIntervalTime(time);
                }
            }

        }else {
            nursingReqVO.setModeId("医疗".equals(modeName) ? "1" : "2");
            nursingRecordsRespVOPage = chDisabAccountsDetailMapper.queryNursingRecords(new Page<>(nursingReqVO.getPageNum(), nursingReqVO.getPageSize()), nursingReqVO);
        }
        return nursingRecordsRespVOPage;
    }

    /**
     * 计算2个时间相差的天数、小时、分钟、秒
     * @param startTime 开始时间
     * @param endTime 截止时间
     * @param format 时间格式 yyyy-MM-dd HH:mm:ss
     * @param str 返回的数据为：day-天、hour-小时、min-分钟、second-秒
     * @return
     */
    public static String dateDiff(String startTime, String endTime,  String format, String str) {
        // 按照传入的格式生成一个simpledateformate对象
        SimpleDateFormat sd = new SimpleDateFormat(format);
        long nd = 1000 * 24 * 60 * 60;// 一天的毫秒数
        long nh = 1000 * 60 * 60;// 一小时的毫秒数
        long nm = 1000 * 60;// 一分钟的毫秒数
        long ns = 1000;// 一秒钟的毫秒数
        long diff;
        long day = 0;
        long hour = 0;
        long min = 0;
        long second = 0;
        // 获得两个时间的毫秒时间差异
        try {
            diff = sd.parse(endTime).getTime() - sd.parse(startTime).getTime();
            // 计算差多少天
            day = diff / nd;
            // 计算差多少小时
            hour = diff / nh ;
            // 计算差多少分钟
            min = diff / nm ;
            // 计算差多少秒
            second = diff / ns;
            // 输出结果
            System.out.println("时间相差：" + day + "天" +
                    (hour - day * 24) + "小时"
                    + (min - day * 24 * 60) + "分钟" +
                    second + "秒。");
            /*System.out.println("hour=" + hour + ",min=" + min);*/
        } catch (ParseException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        String time =  min+"分钟"+ (second - min*60) + "秒";
        return time;
    }

    @Override
    public OverallEduceDecreaseCostRespVO queryOverallCostRecords(OverallCostReqVO req) {
        return chDisabAccountsDetailMapper.queryOverallCostRecords(req);
    }


    @Override
    public Page<OverallCostListRespVO> queryFreeRdInfo(OverallCostReqVO req) {
        req.setDealType((null == req.getDealType() || 0 == req.getDealType()) ? null : req.getDealType());
        Page<OverallCostListRespVO> overallCostListRespVOPage = chDisabFreeRdInfoMapper.queryFreeRdInfo(new Page<>(req.getPageNum(), req.getPageSize()), req);
        return overallCostListRespVOPage;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int overallCostSubmit(OverallCostSubmitReqVO req){
        ChDisabDetailAccounts chDisabDetailAccounts = new ChDisabDetailAccounts();
        int dealType = req.getDealType();
        Long accountsDetailId = req.getAccountsDetailId();
        chDisabDetailAccounts.setAccountsDetailId(accountsDetailId);
        Long accountsId = req.getAccountsId();
        ChDisabAccounts chDisabAccounts = disabAccountsMapper.selectById(accountsId);
        ChDisabDetailAccounts chDisabDetailAccountsDTO = chDisabAccountsDetailMapper.selectById(accountsDetailId);
        BigDecimal deductionAmount = chDisabDetailAccountsDTO.getDeductionAmount();
        BigDecimal reissueAmount = chDisabDetailAccountsDTO.getReissueAmount();
        BigDecimal deductionAmountTotal = chDisabAccounts.getDeductionAmountTotal();
        BigDecimal reissueAmountTotal = chDisabAccounts.getReissueAmountTotal();
        chDisabDetailAccounts.setDealType(dealType);
        String amountOfMoneyStr = req.getAmountOfMoney();
        BigDecimal amountOfMoney = new BigDecimal(amountOfMoneyStr);
        if(dealType == 1){
            // 补发
            chDisabDetailAccounts.setReissueAmount(reissueAmount.add(amountOfMoney));
            reissueAmountTotal = reissueAmountTotal.add(amountOfMoney);
        }else if(dealType == 2){
            // 扣减
            chDisabDetailAccounts.setDeductionAmount(deductionAmount.add(amountOfMoney));
            deductionAmountTotal = deductionAmountTotal.add(amountOfMoney);
        }
        chDisabDetailAccounts.setLcd(new Date());
        int result = chDisabAccountsDetailMapper.updateById(chDisabDetailAccounts);
        chDisabAccounts.setReissueAmountTotal(reissueAmountTotal);
        chDisabAccounts.setDeductionAmountTotal(deductionAmountTotal);
        disabAccountsMapper.updateById(chDisabAccounts);
        ChDisabFreeRdInfo chDisabFreeRdInfo = new ChDisabFreeRdInfo();
        chDisabFreeRdInfo.setDealType(dealType);
        chDisabFreeRdInfo.setAmountOfMoney(amountOfMoney);
        chDisabFreeRdInfo.setAccountsDetailId(accountsDetailId);
        chDisabFreeRdInfo.setAccountsId(accountsId);
        chDisabFreeRdInfo.setCreationTime(new Date());
        chDisabFreeRdInfo.setRemarks(req.getRemarks());
        chDisabFreeRdInfoMapper.insert(chDisabFreeRdInfo);
        return result;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateChDisabAccounts(int id) {
        ChDisabAccounts ca = new ChDisabAccounts();
        ca.setAccountsId(Long.valueOf(id));
        ca.setAccountsStatus((short) 2);
        return disabAccountsMapper.updateById(ca);


    }



    @Override
    public Page<SrvTaskRespVO> getSrvTaskList(SrvTaskReqVO srvTaskReqVO){
        Page<SrvTaskRespVO> srvTaskList = taskDetailMapper.getSrvTaskList(new Page<>(srvTaskReqVO.getPageNum(), srvTaskReqVO.getPageSize()),  srvTaskReqVO);
        return srvTaskList;
    }

    @Override
    public Page<SrvTaskRespVO> getSrvTaskCardList(SrvTaskReqVO srvTaskReqVO){
        Page<SrvTaskRespVO> srvTaskList = taskDetailMapper.getSrvTaskCardList(new Page<>(srvTaskReqVO.getPageNum(), srvTaskReqVO.getPageSize()),  srvTaskReqVO);
        List<String> roleIds = auditingRoleConfig.getRoleId();
        log.info("==========getSrvTaskCardList====roleIds:{}",roleIds);
        // 有审核的权限就配置，根据配置的审核角色查询是否满足条件，如果查到返回给前端1 可以审核，为2不展示审核按钮
        int count = taskDetailMapper.getRoleByUserId(srvTaskReqVO.getUserId(), roleIds);
        List<SrvTaskRespVO> records = srvTaskList.getRecords();
        for (SrvTaskRespVO record : records) {
            if(count>0){
                record.setAuditFlag(true);
            }
        }

        return srvTaskList;
    }


    @Override
    public SrvTaskCardRespDTO getSrvTaskCardInfo(String taskDetailId){
        List<SrvTaskRespVO> records = taskDetailMapper.getSrvTaskCardInfo(taskDetailId);
        List<ItemRespVO> itemRespVOS = new ArrayList<>();
        for (SrvTaskRespVO record : records) {
            ItemRespVO itemRespVO = new ItemRespVO();
            itemRespVO.setItemId(record.getItemId());
            itemRespVO.setItemName(record.getItemName());
            itemRespVOS.add(itemRespVO);
        }
        SrvTaskCardRespDTO srvTaskCardRespDTO = new SrvTaskCardRespDTO();
        srvTaskCardRespDTO.setItemsList(itemRespVOS);
        List<String> collect = records.stream().map(SrvTaskRespVO::getItemName).collect(Collectors.toList());
        String join = StringUtils.join(collect, ",");
        srvTaskCardRespDTO.setItemName(join);
        srvTaskCardRespDTO.setRealName(records.get(0).getRealName());
        srvTaskCardRespDTO.setEmpName(records.get(0).getEmpName());
        srvTaskCardRespDTO.setSrvModeName(records.get(0).getSrvModeName());
        srvTaskCardRespDTO.setSrvOrganName(records.get(0).getSrvOrganName());
        srvTaskCardRespDTO.setSrvDate(records.get(0).getSrvDate());
        srvTaskCardRespDTO.setAuditor(records.get(0).getAuditor());
        srvTaskCardRespDTO.setStatus(records.get(0).getStatus());
        srvTaskCardRespDTO.setReason(records.get(0).getReason());
        srvTaskCardRespDTO.setSrvOrganId(records.get(0).getSrvOrganId());
        srvTaskCardRespDTO.setModeId(records.get(0).getModeId());
        srvTaskCardRespDTO.setEmpId(records.get(0).getEmpId());
        srvTaskCardRespDTO.setAuditRemark(records.get(0).getAuditRemark());
        srvTaskCardRespDTO.setTaskDetailId(records.get(0).getTaskDetailId());
        srvTaskCardRespDTO.setDisabInfoId(records.get(0).getDisabInfoId());
        srvTaskCardRespDTO.setSrvTaskId(Long.parseLong(records.get(0).getSrvTaskId()));
        //查询审核历史记录表数据
        List<SrvTaskDetailAuditLogRespVO> auditLogList=chSrvTaskDetailAuditLogMapper.
                getSrvTaskDetailAuditLogList(records.get(0).getTaskDetailId());
        srvTaskCardRespDTO.setChSrvTaskDetailAuditLogList(auditLogList);//审核记录List
        Result<List<ChFndAttach>> listResult = attachFeignService.selectAttachs(Long.valueOf(taskDetailId), AttachType.DISABLE.getUploadType());
        if (listResult.getCode().equals(ResultCode.SUCCESS.getCode())) {
            srvTaskCardRespDTO.setFileDataList(listResult.getData());
        } else {
            log.info("请求获取附件列表错误：错误码{},错误信息:{}", listResult.getCode(), listResult.getMsg());
        }
        return srvTaskCardRespDTO;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result accountTaskExecute(List<AccountTaskExecuteReqVO> accountTaskExecuteReqVOS) {
        log.info("organAccountTaskExecute.accountTaskExecuteReqVOS:{}",accountTaskExecuteReqVOS);
        if (CollectionUtil.isEmpty(accountTaskExecuteReqVOS)) {
            return Result.failed("参数不能为空");
        }
        // 根据任务id分组，一个任务下会有多个项目，任务id相同
        Map<String, List<AccountTaskExecuteReqVO>> accountTaskMap = accountTaskExecuteReqVOS.stream().collect(Collectors.groupingBy(AccountTaskExecuteReqVO::getTaskDetailId));
        for(Map.Entry<String, List<AccountTaskExecuteReqVO>> entry:accountTaskMap.entrySet()) {
            String taskDetailId = entry.getKey();
            // 已完成的任务,一个任务下有服务完成就产生结算，多个服务完成算一条服务的结算（实际上结算详情有一条记录就产生结算费用）
            List<AccountTaskExecuteReqVO> accountTaskList = entry.getValue();
            // 处理具体的服务项目
            try {

                List<ChSrvTaskFinishItem> chSrvTaskFinishItems = chSrvTaskFinishItemMapper.selectList(new LambdaQueryWrapper<ChSrvTaskFinishItem>()
                        .eq(ChSrvTaskFinishItem::getTaskDetailId, taskDetailId));

                for (AccountTaskExecuteReqVO accountTaskExecuteReqVO : accountTaskList) {
                    // 已申报的任务不允许再执行任务
                    String srvDateYm = DateUtils.dateToStrYm(DateUtils.strToDate(accountTaskExecuteReqVO.getSrvDate()));
                    accountTaskExecuteReqVO.setSrvDateYm(srvDateYm);
                    int count = chDisabAccountsDetailMapper.selectByCondition(accountTaskExecuteReqVO);
                    if(count>0){
                        return Result.failed("此任务已提交申报，不可再操作，请联系管理员");
                    }
                    // 已完成的任务更新ch_srv_plan状态
                    String planId = accountTaskExecuteReqVO.getPlanId();
                    Integer integer = chSrvTaskDetailMapper.updateSrvPlanById(planId);
                    if(integer == 0) {
                        log.info("更新任务计划表失败");
                    }


                    List<ChSrvTaskFinishItem> chSrvTaskFinishItemStream = chSrvTaskFinishItems.stream()
                            .filter(s -> s.getItemId().equals(Long.valueOf(accountTaskExecuteReqVO.getItemId()))).collect(Collectors.toList());
                    // 如果存在就不插入了
                    if (CollectionUtil.isNotEmpty(chSrvTaskFinishItemStream)) {
                        continue;
                    }

                ChSrvTaskFinishItem chSrvTaskFinishItem = new ChSrvTaskFinishItem();
                chSrvTaskFinishItem.setTaskDetailId(Long.valueOf(accountTaskExecuteReqVO.getTaskDetailId()));
                chSrvTaskFinishItem.setItemId(Long.valueOf(accountTaskExecuteReqVO.getItemId()));
                chSrvTaskFinishItemMapper.insert(chSrvTaskFinishItem);

                }
                accountSave1(taskDetailId,accountTaskList);
            } catch (Exception e) {
                log.info("=====accountSave error:{}",e);
                return Result.failed("服务异常");
            }
        }
        return Result.success();
    }


    private void accountSave1(String taskDetailId,List<AccountTaskExecuteReqVO> accountTaskList) {
        // 已完成任务 已结算
        List<OrganAccountsVO> accountsList = chDisabAccountsDetailMapper.queryPlanInfoByDetailId(taskDetailId);
        if (CollectionUtil.isNotEmpty(accountsList)) {
            log.info("该失能人员当天已有执行记录,不在产生费用,taskDetailId:{}",taskDetailId);
            return;
        }else {
            // 不同的护理人员同一天对同一个失能人员服务项目只计一条费用
            TaskDetailDisableVO taskDetailDisableVO = chDisabAccountsDetailMapper.queryTaskDisId(taskDetailId);
            if (Objects.nonNull(taskDetailDisableVO)) {
                int count = chDisabAccountsDetailMapper.queryTaskDetailDisable(taskDetailDisableVO);
                if(count>0){
                    log.info("该失能人员当天已经有护理人员护理过,不在产生费用,taskDetailId:{}",taskDetailId);
                    return;
                }
            }
        }
        ChSrvTaskDetail ca = new ChSrvTaskDetail();
        ca.setStatus((short) 4);
        ca.setLcd(new Date());
        ca.setTaskDetailId(Long.valueOf(taskDetailId));
        chSrvTaskDetailMapper.updateById(ca);
            AccountTaskExecuteReqVO accountsVO = accountTaskList.get(0);
            ChDisabDetailAccounts chDisabDetailAccounts = new ChDisabDetailAccounts();
            String srvOrganId = accountsVO.getSrvOrganId();
            // 结算时间为任务时间
            Date accountsTime = DateUtils.strToDate(accountsVO.getSrvDate());
            String disabInfoId = accountsVO.getDisabInfoId();
            //要判断是养老还是医疗，因为费用不一样
            BigDecimal limit = BigDecimal.valueOf(0);
            BigDecimal overallPercent = BigDecimal.valueOf(0.7);
            BigDecimal personalPercent = BigDecimal.valueOf(0.3);
            String srvModeName = accountsVO.getSrvModeName();
            // 养老机构护理，其限额为 50 元/人/天  由长期护理保险基金支付 70% 个人支付 30%
            if ("养老".equals(srvModeName)) {
                limit = BigDecimal.valueOf(50);
                //  医疗机构护理，原则上收治医疗护理需求为主的重度失能人员一级及以下医疗机构其限额为 80 元/人/天，由长期护理保险基金支付 70%，个人支付 30%;
            } else if ("医疗".equals(srvModeName) && "1".equals(accountsVO.getLevelCode())) {
                limit = BigDecimal.valueOf(80);
                // 二级及以上医疗机构其限额为 100 元/人/天,由长期护理保险基金支付 70%，个人支付 30%;
            } else if ("医疗".equals(srvModeName) && (!"1".equals(accountsVO.getLevelCode()))) {
                limit = BigDecimal.valueOf(100);
            } else if ("上门".equals(srvModeName)) {
                overallPercent = BigDecimal.valueOf(0.8);
                personalPercent = BigDecimal.valueOf(0.2);
                limit = BigDecimal.valueOf(75);
            }
            // 根据护理机构、结算月份、失能人员id和服务方式查询表里是否有值
            ChDisabDetailAccounts chDisabDetailAccountsDTO = chDisabAccountsDetailMapper.queryDisabDetailAccounts(srvOrganId, DateUtils.dateToStrYm(accountsTime), disabInfoId, srvModeName);
            BigDecimal overallCost = limit.multiply(overallPercent);
            BigDecimal personalCost = limit.multiply(personalPercent);
            if(Objects.isNull(chDisabDetailAccountsDTO)){
                chDisabDetailAccounts.setAccountsAllCost(limit);
                chDisabDetailAccounts.setAccountsOverallCost(overallCost);
                chDisabDetailAccounts.setAccountsPersonalCost(personalCost);
                chDisabDetailAccounts.setModeName(accountsVO.getSrvModeName());
                chDisabDetailAccounts.setSrvOrganId(srvOrganId);
                chDisabDetailAccounts.setCreationTime(new Date());
                chDisabDetailAccounts.setAccountsTime(accountsTime);
                chDisabDetailAccounts.setAccountsStartTime(DateUtils.getFirstDayOfMonth(DateUtils.strToDate(accountsVO.getSrvDate())));
                chDisabDetailAccounts.setAccountsEndTime(DateUtils.getlastDayOfMonth(DateUtils.strToDate(accountsVO.getSrvDate())));
                chDisabDetailAccounts.setMedicalArea(accountsVO.getMedicalArea());
                chDisabDetailAccounts.setDisabInfoId(disabInfoId);
                chDisabDetailAccounts.setStatus((short)0);
                chDisabDetailAccounts.setReissueAmount(BigDecimal.valueOf(0));
                chDisabDetailAccounts.setDeductionAmount(BigDecimal.valueOf(0));
                ChDisabAccounts chDisabAccounts = getChDisabAccounts1(accountsVO);
                chDisabDetailAccounts.setAccountsId(chDisabAccounts.getAccountsId());
                chDisabAccountsDetailMapper.insert(chDisabDetailAccounts);
            }else {
                BigDecimal accountsAllCost = chDisabDetailAccountsDTO.getAccountsAllCost();
                BigDecimal accountsOverallCost = chDisabDetailAccountsDTO.getAccountsOverallCost();
                BigDecimal accountsPersonalCost = chDisabDetailAccountsDTO.getAccountsPersonalCost();
                chDisabDetailAccountsDTO.setAccountsAllCost(accountsAllCost.add(limit));
                chDisabDetailAccountsDTO.setAccountsOverallCost(accountsOverallCost.add(overallCost));
                chDisabDetailAccountsDTO.setAccountsPersonalCost(accountsPersonalCost.add(personalCost));
                chDisabDetailAccountsDTO.setLcd(new Date());
                chDisabAccountsDetailMapper.updateById(chDisabDetailAccountsDTO);
            }

    }


    private ChDisabAccounts getChDisabAccounts1(AccountTaskExecuteReqVO accountsVO) {
        // 结算表  根据机构和结算时间  服务方式
        ChDisabAccounts chDisabAccounts = disabAccountsMapper.queryAccountByCondition(DateUtils.dateToStrYm(DateUtils.strToDate(accountsVO.getSrvDate())), accountsVO.getSrvModeName(), accountsVO.getSrvOrganId(),null);
        if(chDisabAccounts == null) {
            chDisabAccounts = new ChDisabAccounts();
            chDisabAccounts.setModeName(accountsVO.getSrvModeName());
            chDisabAccounts.setAccountsTime(DateUtils.strToDateMd(accountsVO.getSrvDate()));
            chDisabAccounts.setSrvOrganId(Long.valueOf(accountsVO.getSrvOrganId()));
            chDisabAccounts.setCreationTime(new Date());
            chDisabAccounts.setAccountsStatus((short) 0);
            BigDecimal zero = BigDecimal.valueOf(0);
            chDisabAccounts.setAllCost(zero);
            chDisabAccounts.setOverallCost(zero);
            chDisabAccounts.setPersonalCost(zero);
            chDisabAccounts.setReissueAmountTotal(zero);
            chDisabAccounts.setDeductionAmountTotal(zero);
            disabAccountsMapper.insert(chDisabAccounts);
        }
        return chDisabAccounts;
    }


        @Override
        public void otherPlaceHandler(String programId){
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int deleteAccountsDetail(Long accountsDetailId,Long freeRdId){
        ChDisabFreeRdInfo chDisabFreeRdInfo = chDisabFreeRdInfoMapper.selectById(freeRdId);
        BigDecimal amountOfMoney = chDisabFreeRdInfo.getAmountOfMoney();
        int freeDealType = chDisabFreeRdInfo.getDealType();
        ChDisabDetailAccounts chDisabDetailAccountsDTO = chDisabAccountsDetailMapper.selectById(accountsDetailId);
        BigDecimal deductionAmount = chDisabDetailAccountsDTO.getDeductionAmount();
        BigDecimal reissueAmount = chDisabDetailAccountsDTO.getReissueAmount();
        Long accountsId = chDisabDetailAccountsDTO.getAccountsId();
        ChDisabAccounts chDisabAccountDTO = disabAccountsMapper.selectById(accountsId);
        BigDecimal reissueAmountTotal = chDisabAccountDTO.getReissueAmountTotal();
        BigDecimal deductionAmountTotal = chDisabAccountDTO.getDeductionAmountTotal();
        if (freeDealType == 1) {
            // 删除减掉补发
            BigDecimal reissueAmountSubtract = reissueAmountTotal.subtract(amountOfMoney);
            chDisabAccountDTO.setReissueAmountTotal(reissueAmountSubtract);
            BigDecimal subtract = reissueAmount.subtract(amountOfMoney);
            chDisabDetailAccountsDTO.setReissueAmount(subtract);
        } else if (freeDealType == 2) {
            // 删除减掉扣减
            BigDecimal deductionAmountSubtract = deductionAmountTotal.subtract(amountOfMoney);
            chDisabAccountDTO.setDeductionAmountTotal(deductionAmountSubtract);
            BigDecimal subtract = deductionAmount.subtract(amountOfMoney);
            chDisabDetailAccountsDTO.setDeductionAmount(subtract);
        }
        disabAccountsMapper.updateById(chDisabAccountDTO);
        chDisabDetailAccountsDTO.setLcd(new Date());
        int result = chDisabAccountsDetailMapper.updateById(chDisabDetailAccountsDTO);
        chDisabFreeRdInfoMapper.deleteById(freeRdId);
        return result;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result accountsFreeCommit(Long accountsDetailId){
        log.info("accountsFreeCommit=========accountsDetailId:{}",accountsDetailId);
        ChDisabDetailAccounts chDisabDetailAccountsDTO = chDisabAccountsDetailMapper.selectById(accountsDetailId);
        log.info("accountsFreeCommit=========chDisabDetailAccountsDTO:{}",JSON.toJSONString(chDisabDetailAccountsDTO));
        BigDecimal allCost = chDisabDetailAccountsDTO.getAccountsAllCost();
        BigDecimal overallCost  = chDisabDetailAccountsDTO.getAccountsOverallCost();
        BigDecimal personalCost = chDisabDetailAccountsDTO.getAccountsPersonalCost();
        Short status = chDisabDetailAccountsDTO.getStatus();
        if(status == 1 ){
            log.info("{},已完成申报", accountsDetailId);
            return Result.failed("已完成申报，请勿重复提交");
        }
        Long accountsId = chDisabDetailAccountsDTO.getAccountsId();
        ChDisabAccounts chDisabAccountDTO = disabAccountsMapper.selectById(accountsId);
        log.info("accountsFreeCommit====before=====chDisabAccountDTO:{}",JSON.toJSONString(chDisabAccountDTO));
        Short accountsStatus = chDisabAccountDTO.getAccountsStatus();
        if(accountsStatus > 1){
            log.info("{},已完成审核", accountsDetailId);
            return Result.failed("机构已完成审核，不能再进行申报");
        }

        if(chDisabAccountDTO !=null){
            BigDecimal accountAllCost = chDisabAccountDTO.getAllCost();
            BigDecimal accountOverallCost = chDisabAccountDTO.getOverallCost();
            BigDecimal accountPersonalCost = chDisabAccountDTO.getPersonalCost();
            BigDecimal addAllCost = accountAllCost.add(allCost);
            chDisabAccountDTO.setAllCost(addAllCost);
            chDisabAccountDTO.setOverallCost(accountOverallCost.add(overallCost));
            chDisabAccountDTO.setPersonalCost(accountPersonalCost.add(personalCost));
            chDisabAccountDTO.setAccountsStatus((short)1);
            log.info("accountsFreeCommit====accountsDetailId:{},累加前费用:{},申报费用:{},累加后费用:{}",accountsDetailId,accountAllCost,allCost,addAllCost);
            disabAccountsMapper.updateById(chDisabAccountDTO);
        }
        ChDisabDetailAccounts chDisabDetailAccounts = new ChDisabDetailAccounts();
        chDisabDetailAccounts.setAccountsDetailId(accountsDetailId);
        chDisabDetailAccounts.setStatus(((short)1));
        chDisabDetailAccounts.setDeclarationTime(new Date());
        chDisabDetailAccounts.setLcd(new Date());
        log.info("accountsFreeCommit=====after====chDisabDetailAccountsDTO:{}",JSON.toJSONString(chDisabDetailAccounts));
        int result = chDisabAccountsDetailMapper.updateById(chDisabDetailAccounts);
        if (result > 0) {
            return Result.success();
        } else {
            return Result.failed("操作失败");
        }
    }




    @Override
    public int delayAccountCommit(Long accountsDetailId,Short status){
        // 暂缓需要将当月的钱减掉
        ChDisabDetailAccounts chDisabDetailAccountDTO = chDisabAccountsDetailMapper.selectById(accountsDetailId);
        log.info("delayAccountCommit====before=====chDisabDetailAccountDTO:{}",JSON.toJSONString(chDisabDetailAccountDTO));
        Long accountsId = chDisabDetailAccountDTO.getAccountsId();
        BigDecimal accountsAllCost = chDisabDetailAccountDTO.getAccountsAllCost();
        BigDecimal accountsOverallCost = chDisabDetailAccountDTO.getAccountsOverallCost();
        BigDecimal accountsPersonalCost = chDisabDetailAccountDTO.getAccountsPersonalCost();
        BigDecimal deductionAmount = chDisabDetailAccountDTO.getDeductionAmount();
        BigDecimal reissueAmount = chDisabDetailAccountDTO.getReissueAmount();
        ChDisabAccounts chDisabAccountDTO = disabAccountsMapper.selectById(accountsId);
        log.info("delayAccountCommit====before=====chDisabAccountDTO:{}",JSON.toJSONString(chDisabAccountDTO));
        BigDecimal allCost = chDisabAccountDTO.getAllCost();
        BigDecimal overallCost = chDisabAccountDTO.getOverallCost();
        BigDecimal personalCost = chDisabAccountDTO.getPersonalCost();
        BigDecimal deductionAmountTotal = chDisabAccountDTO.getDeductionAmountTotal();
        BigDecimal reissueAmountTotal = chDisabAccountDTO.getReissueAmountTotal();
        Short accountsStatus = chDisabAccountDTO.getAccountsStatus();
        if (accountsStatus > 1) {
            log.info("{},已完成审核",accountsDetailId);
            return -1;
        }
        // 暂缓
        if(status == 2){
            allCost = allCost.subtract(accountsAllCost);
            overallCost = overallCost.subtract(accountsOverallCost);
            personalCost = personalCost.subtract(accountsPersonalCost);
            BigDecimal deductionAmountTotalSub = deductionAmountTotal.subtract(deductionAmount);
            BigDecimal reissueAmountTotalSub = reissueAmountTotal.subtract(reissueAmount);
            ChDisabAccounts chDisabAccount = new ChDisabAccounts();
            chDisabAccount.setAllCost(allCost);
            chDisabAccount.setOverallCost(overallCost);
            chDisabAccount.setPersonalCost(personalCost);
            chDisabAccount.setAccountsId(accountsId);
            chDisabAccount.setDeductionAmountTotal(deductionAmountTotalSub);
            chDisabAccount.setReissueAmountTotal(reissueAmountTotalSub);
            disabAccountsMapper.updateById(chDisabAccount);
            // 恢复
        }else if(status == 1){
            // 判断恢复时间是否和之前结算时间在同一个月，如果在同一个月把金额加回来，如果不在同一个月，改变结算月份,恢复的时候重新插入一条
            //结算记录
            Date accountsTime = chDisabDetailAccountDTO.getAccountsTime();
            Boolean aBoolean = DateUtils.inCurrentMonth(accountsTime);
            //  同一个月
            if(aBoolean){
                allCost = allCost.add(accountsAllCost);
                overallCost = overallCost.add(accountsOverallCost);
                personalCost = personalCost.add(accountsPersonalCost);
                BigDecimal reissueAmountAdd = reissueAmountTotal.add(reissueAmount);
                BigDecimal deductionAmountAdd = deductionAmountTotal.add(deductionAmount);
                ChDisabAccounts chDisabAccount = new ChDisabAccounts();
                chDisabAccount.setAllCost(allCost);
                chDisabAccount.setOverallCost(overallCost);
                chDisabAccount.setPersonalCost(personalCost);
                chDisabAccount.setAccountsId(accountsId);
                chDisabAccount.setReissueAmountTotal(reissueAmountAdd);
                chDisabAccount.setDeductionAmountTotal(deductionAmountAdd);
                disabAccountsMapper.updateById(chDisabAccount);
            }else {
                // 结算到其他月份 结算表加一个是否暂缓字段 2 暂缓
                ChDisabAccounts chDisabAccounts = disabAccountsMapper.queryAccountByCondition(DateUtils.dateToStrYm(new Date()), chDisabDetailAccountDTO.getModeName(), chDisabDetailAccountDTO.getSrvOrganId(),2);
                if (chDisabAccounts == null) {
                    chDisabAccounts = new ChDisabAccounts();
                    chDisabAccounts.setIsDelay((short)2);
                    chDisabAccounts.setAccountsStatus((short)1);
                    chDisabAccounts.setSrvOrganId(Long.valueOf(chDisabDetailAccountDTO.getSrvOrganId()));
                    chDisabAccounts.setCreationTime(new Date());
                    chDisabAccounts.setAccountsTime(new Date());
                    chDisabAccounts.setModeName(chDisabDetailAccountDTO.getModeName());
                    chDisabAccounts.setDeductionAmountTotal(chDisabDetailAccountDTO.getDeductionAmount());
                    chDisabAccounts.setReissueAmountTotal(chDisabDetailAccountDTO.getReissueAmount());
                    chDisabAccounts.setAllCost(chDisabDetailAccountDTO.getAccountsAllCost());
                    chDisabAccounts.setOverallCost(chDisabDetailAccountDTO.getAccountsOverallCost());
                    chDisabAccounts.setPersonalCost(chDisabDetailAccountDTO.getAccountsPersonalCost());
                    chDisabAccounts.setSvrStartTime(chDisabDetailAccountDTO.getAccountsStartTime());
                    chDisabAccounts.setSvrEndTime(chDisabDetailAccountDTO.getAccountsEndTime());
                    chDisabAccounts.setTotalAllocatedExpenses(BigDecimal.valueOf(0));
                    disabAccountsMapper.insert(chDisabAccounts);
                    accountsId = chDisabAccounts.getAccountsId();
                }else {
                    BigDecimal allCost1 = chDisabAccounts.getAllCost();
                    BigDecimal overallCost1 = chDisabAccounts.getOverallCost();
                    BigDecimal personalCost1 = chDisabAccounts.getPersonalCost();
                    BigDecimal deductionAmountTotal1 = chDisabAccounts.getDeductionAmountTotal();
                    BigDecimal reissueAmountTotal1 = chDisabAccounts.getReissueAmountTotal();
                    BigDecimal deduction = deductionAmountTotal1.add(deductionAmount);
                    BigDecimal reissue = reissueAmountTotal1.add(reissueAmount);
                    allCost1 = allCost1.add(accountsAllCost);
                    overallCost1 = overallCost1.add(accountsOverallCost);
                    personalCost1 = personalCost1.add(accountsPersonalCost);
                    ChDisabAccounts chDisabAccount = new ChDisabAccounts();
                    chDisabAccount.setAllCost(allCost1);
                    chDisabAccount.setOverallCost(overallCost1);
                    chDisabAccount.setPersonalCost(personalCost1);
                    chDisabAccount.setAccountsId(accountsId);
                    chDisabAccount.setReissueAmountTotal(reissue);
                    chDisabAccount.setDeductionAmountTotal(deduction);
                    disabAccountsMapper.updateById(chDisabAccount);
                }
            }
        }
        ChDisabDetailAccounts chDisabDetailAccounts = new ChDisabDetailAccounts();
        chDisabDetailAccounts.setAccountsDetailId(accountsDetailId);
        chDisabDetailAccounts.setStatus(status);
        chDisabDetailAccounts.setAccountsId(accountsId);
        chDisabDetailAccounts.setLcd(new Date());
        int result = chDisabAccountsDetailMapper.updateById(chDisabDetailAccounts);
        return result;
    }

    @Override
    public Result placeOtherApply(Integer disabInfoId, String otherApplyTime) {
            Result result1 =Result.success();
            Date date = DateUtils.strToDate(otherApplyTime);
            boolean now = DateUtils.isNow(date);
            if (now) {
                  result1 = placeFreeHandler(disabInfoId, otherApplyTime);
            }

            if(result1.equals(Result.success())) {
                int result = taskDetailMapper.placeOtherApply(disabInfoId, otherApplyTime);
                if (result > 0) {
                    return result1;
                } else {
                    return Result.failed("申请失败");
                }
            }
            return result1;
    }


    private Result placeFreeHandler(Integer disabInfoId, String otherApplyTime) {
        BigDecimal limit = BigDecimal.valueOf(20);
        BigDecimal zero = BigDecimal.valueOf(0);
        ChDisabDetailAccounts chDisabDetailAccountsDTO = chDisabAccountsDetailMapper.queryDisabDetailAccounts("1361", otherApplyTime, String.valueOf(disabInfoId), "异地");
        if (Objects.isNull(chDisabDetailAccountsDTO)) {
            // 异地
            ChDisabDetailAccounts chDisabDetailAccounts = new ChDisabDetailAccounts();
            chDisabDetailAccounts.setAccountsAllCost(limit);
            chDisabDetailAccounts.setAccountsOverallCost(limit);
            chDisabDetailAccounts.setAccountsPersonalCost(zero);
            chDisabDetailAccounts.setModeName("异地");
            chDisabDetailAccounts.setSrvOrganId("1361");
            chDisabDetailAccounts.setCreationTime(new Date());
            chDisabDetailAccounts.setAccountsTime(DateUtils.strToDate(otherApplyTime));
            chDisabDetailAccounts.setDisabInfoId(String.valueOf(disabInfoId));
            chDisabDetailAccounts.setStatus((short) 1);
            chDisabDetailAccounts.setReissueAmount(BigDecimal.valueOf(0));
            chDisabDetailAccounts.setDeductionAmount(BigDecimal.valueOf(0));
            ChDisabAccounts chDisabAccounts = disabAccountsMapper.queryAccountByCondition(DateUtils.dateToStrYm(DateUtils.strToDate(otherApplyTime)), "异地", "1361", null);
            if (chDisabAccounts == null) {
                chDisabAccounts = new ChDisabAccounts();
                chDisabAccounts.setModeName("异地");
                chDisabAccounts.setAccountsTime(DateUtils.strToDateMd(otherApplyTime));
                chDisabAccounts.setSrvOrganId(Long.valueOf(1361));
                chDisabAccounts.setCreationTime(new Date());
                chDisabAccounts.setAccountsStatus((short) 1);
                chDisabAccounts.setAllCost(limit);
                chDisabAccounts.setOverallCost(limit);
                chDisabAccounts.setPersonalCost(zero);
                chDisabAccounts.setReissueAmountTotal(zero);
                chDisabAccounts.setDeductionAmountTotal(zero);
                disabAccountsMapper.insert(chDisabAccounts);
            }else {
                Short accountsStatus = chDisabAccounts.getAccountsStatus();
                if(accountsStatus > 1){
                    log.info("{},已完成审核",chDisabAccounts.getAccountsId());
                    return Result.failed("提交月份已完成审核,请联系管理员");
                }
                BigDecimal allCost1 = chDisabAccounts.getAllCost();
                BigDecimal overallCost1 = chDisabAccounts.getOverallCost();
                allCost1 = allCost1.add(limit);
                overallCost1 = overallCost1.add(limit);
                ChDisabAccounts chDisabAccount = new ChDisabAccounts();
                chDisabAccount.setAllCost(allCost1);
                chDisabAccount.setOverallCost(overallCost1);
                Long accountsId = chDisabAccounts.getAccountsId();
                chDisabAccount.setAccountsId(accountsId);
                disabAccountsMapper.updateById(chDisabAccount);
            }
            chDisabDetailAccounts.setAccountsId(chDisabAccounts.getAccountsId());
            chDisabAccountsDetailMapper.insert(chDisabDetailAccounts);
        } else {
            BigDecimal accountsAllCost = chDisabDetailAccountsDTO.getAccountsAllCost();
            BigDecimal accountsOverallCost = chDisabDetailAccountsDTO.getAccountsOverallCost();
            chDisabDetailAccountsDTO.setAccountsAllCost(accountsAllCost.add(limit));
            chDisabDetailAccountsDTO.setAccountsOverallCost(accountsOverallCost.add(limit));
            chDisabDetailAccountsDTO.setAccountsPersonalCost(zero);
            chDisabDetailAccountsDTO.setLcd(new Date());
            chDisabAccountsDetailMapper.updateById(chDisabDetailAccountsDTO);
        }
        return Result.success();
    }

    @Override
    public Page<PlaceOtherVO> placeOtherQuery(Page page, String name, String certiCode) {
        Page<PlaceOtherVO> placeOtherVOPage = taskDetailMapper.placeOtherQuery(page, name, certiCode);
        return placeOtherVOPage;
    }



    @Override
    @Transactional(rollbackFor = Exception.class)
    public void placeOtherTaskQuery(){
        List<PlaceOtherVO> placeOtherVOS = taskDetailMapper.placeOtherTaskQuery();
        if(CollectionUtil.isNotEmpty(placeOtherVOS)) {
            int num = placeOtherVOS.size();
            int days = DateUtils.getDays();
            int total = days*20;
            BigDecimal limit = BigDecimal.valueOf(total);
            BigDecimal zero = BigDecimal.valueOf(0);
            ChDisabAccounts chDisabAccounts = updateOtherAccounts(num, limit, zero);
            for (PlaceOtherVO placeOtherVO : placeOtherVOS) {
                Integer disabInfoId = placeOtherVO.getDisabInfoId();
                Long accountsId = chDisabAccounts.getAccountsId();
                placeFreeHandler1(disabInfoId,accountsId,limit,zero);
            }
        }
    }




    private Result placeFreeHandler1(Integer disabInfoId,Long accountsId,BigDecimal limit,BigDecimal zero ) {
        ChDisabDetailAccounts chDisabDetailAccountsDTO = chDisabAccountsDetailMapper.queryDisabDetailAccounts("1361", DateUtils.dateToStrYm(new Date()), String.valueOf(disabInfoId), "异地");
        if (Objects.isNull(chDisabDetailAccountsDTO)) {
            // 异地
            ChDisabDetailAccounts chDisabDetailAccounts = new ChDisabDetailAccounts();
            chDisabDetailAccounts.setAccountsAllCost(limit);
            chDisabDetailAccounts.setAccountsOverallCost(limit);
            chDisabDetailAccounts.setAccountsPersonalCost(zero);
            chDisabDetailAccounts.setModeName("异地");
            chDisabDetailAccounts.setSrvOrganId("1361");
            chDisabDetailAccounts.setCreationTime(new Date());
            chDisabDetailAccounts.setAccountsTime(new Date());
            chDisabDetailAccounts.setDisabInfoId(String.valueOf(disabInfoId));
            chDisabDetailAccounts.setStatus((short) 1);
            chDisabDetailAccounts.setReissueAmount(BigDecimal.valueOf(0));
            chDisabDetailAccounts.setDeductionAmount(BigDecimal.valueOf(0));
            chDisabDetailAccounts.setAccountsId(accountsId);
            chDisabAccountsDetailMapper.insert(chDisabDetailAccounts);
        } else {
            chDisabDetailAccountsDTO.setAccountsAllCost(limit);
            chDisabDetailAccountsDTO.setAccountsOverallCost(limit);
            chDisabDetailAccountsDTO.setAccountsPersonalCost(zero);
            chDisabDetailAccountsDTO.setLcd(new Date());
            chDisabAccountsDetailMapper.updateById(chDisabDetailAccountsDTO);

        }
        return Result.success();
    }

    private ChDisabAccounts updateOtherAccounts(int num, BigDecimal limit, BigDecimal zero) {
        ChDisabAccounts chDisabAccounts = disabAccountsMapper.queryAccountByCondition(DateUtils.dateToStrYm(new Date()), "异地", "1361", null);
        BigDecimal numBig = new BigDecimal(num);
        BigDecimal cost = limit.multiply(numBig);
        if (chDisabAccounts == null) {
            chDisabAccounts = new ChDisabAccounts();
            chDisabAccounts.setModeName("异地");
            chDisabAccounts.setAccountsTime(new Date());
            chDisabAccounts.setSrvOrganId(Long.valueOf(1361));
            chDisabAccounts.setCreationTime(new Date());
            chDisabAccounts.setAccountsStatus((short) 1);
            chDisabAccounts.setAllCost(cost);
            chDisabAccounts.setOverallCost(cost);
            chDisabAccounts.setPersonalCost(zero);
            chDisabAccounts.setReissueAmountTotal(zero);
            chDisabAccounts.setDeductionAmountTotal(zero);
            disabAccountsMapper.insert(chDisabAccounts);
        }else {
            ChDisabAccounts chDisabAccount = new ChDisabAccounts();
            chDisabAccount.setAllCost(cost);
            chDisabAccount.setOverallCost(cost);
            Long accountsId = chDisabAccounts.getAccountsId();
            chDisabAccount.setAccountsId(accountsId);
            disabAccountsMapper.updateById(chDisabAccount);
        }
        return chDisabAccounts;
    }


    /**
     *@desc 退回
     * 1、根据服务机构，时间，失能人员，服务方式查询结算明细表
     * 2、删除完成项目表的数据
     * 3、查询计划表是否还有已完成的任务，有就只更新状态，
     * 只有一条记录结算明细表需要退回费用，并更新服务计划详情表状态
     **/
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result accountReturn(AccountTaskExecuteReqVO srvTaskRespVO){
        Result result = accountReturnHandler(srvTaskRespVO);
        return result;
    }


    public  Result accountReturnHandler(AccountTaskExecuteReqVO srvTaskRespVO) {
        Long taskDetailId = Long.valueOf(srvTaskRespVO.getTaskDetailId());
        String disabInfoId = srvTaskRespVO.getDisabInfoId();
        String srvModeName = srvTaskRespVO.getSrvModeName();
        String srvPlanTime = srvTaskRespVO.getSrvDate();
        Date date = DateUtils.strToDate(srvPlanTime);
        String srvOrganId = srvTaskRespVO.getSrvOrganId();
        String itemId = srvTaskRespVO.getItemId();
        log.info("AccountServiceImpl====accountReturn===taskDetailId:{},disabInfoId:{},srvPlanTime:{},itemId:{}", taskDetailId, disabInfoId, srvPlanTime, itemId);
        ChDisabDetailAccounts chDisabDetailAccounts = chDisabAccountsDetailMapper.queryDisabDetailAccounts(String.valueOf(srvOrganId), DateUtils.dateToStrYm(date), disabInfoId, srvModeName);
        Short status = chDisabDetailAccounts.getStatus();
        if (status != 0) {
            return Result.failed("此任务已提交申报，不可再操作，请联系管理员");
        }
        // 删除  chSrvTaskFinishItemStream
        LambdaQueryWrapper<ChSrvTaskFinishItem> chSrvTaskFinishItem = new LambdaQueryWrapper<ChSrvTaskFinishItem>()
                .eq(ChSrvTaskFinishItem::getTaskDetailId, taskDetailId)
                .eq(ChSrvTaskFinishItem::getItemId, itemId);
        int delete = chSrvTaskFinishItemMapper.delete(chSrvTaskFinishItem);
        LambdaQueryWrapper<ChSrvPlan> queryWrapper = new LambdaQueryWrapper<>();
        // 一个失能人员对应多个护理人员的情况，需要用时间统计
        //queryWrapper.eq(ChSrvPlan::getTaskDetailId,taskDetailId);
        queryWrapper.eq(ChSrvPlan::getSrvPlanTime, date);
        queryWrapper.eq(ChSrvPlan::getDisabInfoId, disabInfoId);
        queryWrapper.eq(ChSrvPlan::getIsStatus, (short) 4);
        List<ChSrvPlan> srvPlanList = chSrvPlanMapper.selectList(queryWrapper);
        int size = srvPlanList.size();
        log.info("accountReturn===paln===size:{}", size);
        // 大于1说明还有已完成的任务,不退回,只更新状态 等于1结算退回费用

        ChSrvPlan chSrvPlan = new ChSrvPlan();
        chSrvPlan.setIsStatus(0L);
        chSrvPlan.setIsBill(0L);
        chSrvPlan.setLcd(new Date());
        chSrvPlan.setPlanID(Long.valueOf(srvTaskRespVO.getPlanId()));
        chSrvPlanMapper.updateById(chSrvPlan);
        if (size == 1) {
            BigDecimal accountsAllCost = chDisabDetailAccounts.getAccountsAllCost();
            if (accountsAllCost.compareTo(BigDecimal.ZERO) == 0) {
                return Result.success("当前费用为0，不再进行费用退回");
            }
            BigDecimal accountsOverallCost = chDisabDetailAccounts.getAccountsOverallCost();
            BigDecimal accountsPersonalCost = chDisabDetailAccounts.getAccountsPersonalCost();
            BigDecimal allCost = new BigDecimal(0);
            BigDecimal overCost = new BigDecimal(0);
            BigDecimal personCost = new BigDecimal(0);
            if ("养老".equals(srvModeName)) {
                allCost = BigDecimal.valueOf(50);
                overCost = BigDecimal.valueOf(35);
                personCost = BigDecimal.valueOf(15);
                //  医疗机构护理，原则上收治医疗护理需求为主的重度失能人员一级及以下医疗机构其限额为 80 元/人/天，由长期护理保险基金支付 70%，个人支付 30%;
            } else if ("医疗".equals(srvModeName) && "1".equals(srvTaskRespVO.getLevelCode())) {
                allCost = BigDecimal.valueOf(80);
                overCost = BigDecimal.valueOf(56);
                personCost = BigDecimal.valueOf(24);
                // 二级及以上医疗机构其限额为 100 元/人/天,由长期护理保险基金支付 70%，个人支付 30%;
            } else if ("医疗".equals(srvModeName) && (!"1".equals(srvTaskRespVO.getLevelCode()))) {
                allCost = BigDecimal.valueOf(100);
                overCost = BigDecimal.valueOf(70);
                personCost = BigDecimal.valueOf(30);
            } else if ("上门".equals(srvModeName)) {
                allCost = BigDecimal.valueOf(75);
                overCost = BigDecimal.valueOf(60);
                personCost = BigDecimal.valueOf(15);
            }
            BigDecimal allCostSubtract = accountsAllCost.subtract(allCost);
            BigDecimal overCostSubtract = accountsOverallCost.subtract(overCost);
            BigDecimal personCostSubtract = accountsPersonalCost.subtract(personCost);
            chDisabDetailAccounts.setAccountsAllCost(allCostSubtract);
            chDisabDetailAccounts.setAccountsOverallCost(overCostSubtract);
            chDisabDetailAccounts.setAccountsPersonalCost(personCostSubtract);
            chDisabDetailAccounts.setLcd(new Date());
            chDisabAccountsDetailMapper.updateById(chDisabDetailAccounts);
            ChSrvTaskDetail ca = new ChSrvTaskDetail();
            ca.setStatus((short) 0);
            ca.setTaskDetailId(Long.valueOf(taskDetailId));
            ca.setLcd(new Date());
            chSrvTaskDetailMapper.updateById(ca);
        }
      return    Result.success();
    }



    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result batchAccountReturn(List<AccountTaskExecuteReqVO> srvTaskListRespVO){
        try {
        for (AccountTaskExecuteReqVO srvTaskRespVO : srvTaskListRespVO) {
            Result result = accountReturnHandler(srvTaskRespVO);
            if(!ResultCode.SUCCESS.getCode().equals(result.getCode())){
                return result;
            }
        }
        } catch (Exception e) {
            log.error("==============accountReturn error:{}",e);
            return  Result.failed("服务器异常");
        }
        return Result.success();
    }

}


