package com.hungraim.ltc.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ltc.pojo.entity.disable.ChDisableApply;
import com.hungraim.ltc.pojo.entity.facelocation.FaceLocationWhiteList;
import com.hungraim.ltc.pojo.vo.disable.DisableUpdateDetailVO;
import com.hungraim.ltc.pojo.vo.facelocation.FaceLocationWhiteListVo;
import com.hungraim.ltc.service.FaceLocationWhiteListService;
import com.hungraim.ltc.util.Result;
import com.hungraim.ltc.util.ResultCode;
import com.hungraim.ltc.util.TokenParsingUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * 人脸识别和定位白名单Controller
 *
 * @author czz
 */
@RestController
@RequestMapping("/api.apply/facelocation")
@Slf4j
public class FaceLocationWhiteListController {

    @Autowired
    private FaceLocationWhiteListService faceLocationWhiteListService;

    /**
     * 失能人员信息退出列表
     *
     * @param pageNum            当前页
     * @param pageSize           每页几条数据
     * @return 失能人员信息列表
     */
    @RequestMapping("/whiteList")
    public Result<Page<FaceLocationWhiteListVo>> whiteList(@RequestHeader("Authorization")String token,
                                                           Integer pageNum,
                                                           Integer pageSize, Long srvOrganId,
                                                           String certiCode, String realName,
                                                           String empName) {
        if (pageNum == null || pageNum < 1) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (pageSize == null || pageSize < 1) {
            pageSize = 10;
        }
        long srvOrganIdL = TokenParsingUtil.TokenParsing(token, "srvOrganId");
        if(srvOrganIdL != 0) {
            srvOrganId = new Long(srvOrganIdL);
        }
        Page<FaceLocationWhiteListVo> page = new Page<>(pageNum, pageSize);
        page = faceLocationWhiteListService.getFaceLocationWhiteList(page,
                srvOrganId,certiCode,realName,empName);
        return Result.success(page);
    }


    /**
     * 白名单详情
     * @param listId 详情id
     * @return
     */
    @RequestMapping("/getFaceLocationWhiteDetail")
    public Result<FaceLocationWhiteListVo> getFaceLocationWhiteDetail(Long listId) {
        if (null == listId) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        FaceLocationWhiteListVo disableUpdateDetailVO = faceLocationWhiteListService.getFaceLocationWhiteDetail(listId);
        return Result.success(disableUpdateDetailVO);
    }

    /**
     * 新增人脸识别定位配置
     */
    @RequestMapping("/addFaceLocationWhiteList")
    public Result addFaceLocationWhiteList(@RequestHeader("Authorization") String token, FaceLocationWhiteList faceLocationWhiteList) {

        if (faceLocationWhiteList.getSrvOrganId()==null) {
            return Result.failed("护理机构不能为空!");
        }
        if (faceLocationWhiteList.getDisabInfoId()==null) {
            return Result.failed("失能人员不能为空!");
        }
        if (faceLocationWhiteList.getEmpId()== null) {
            return Result.failed("护理人员不能为空!");
        }
        if (StringUtils.isBlank(faceLocationWhiteList.getIsPrecisePositioning())) {
            return Result.failed("是否精确定位配置的距离不能为空!");
        }
        if (StringUtils.isBlank(faceLocationWhiteList.getIsFace())) {
            return Result.failed("是否人脸识别不能为空!");
        }
        if (faceLocationWhiteList.getPositioningDistance()== null) {
            return Result.failed("定位距离不能为空!");
        }
        Integer userId = TokenParsingUtil.TokenParsing(token, "userId");
        if(faceLocationWhiteList.getListId()==null){//如果已经新增过，则无需修改新增人，只需要修改人变更
            faceLocationWhiteList.setFcu(userId);
        }
        faceLocationWhiteList.setLcu(userId);
        return faceLocationWhiteListService.addOrUpdateFaceLocationWhiteList(faceLocationWhiteList);
    }

    @RequestMapping("/deleteFaceLocationWhiteListById")
    public Result deleteFaceLocationWhiteListById(@RequestHeader("Authorization") String token, Long listId) {
        return faceLocationWhiteListService.deleteFaceLocationWhiteListById(listId);
    }

}
