package com.hungraim.ltc.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hungraim.ltc.api.OrganFeignService;
import com.hungraim.ltc.dao.*;
import com.hungraim.ltc.pojo.entity.assessment.ChAprTask;
import com.hungraim.ltc.pojo.entity.assessment.ChDisableInfoHis;
import com.hungraim.ltc.pojo.entity.disable.ChDisableApply;
import com.hungraim.ltc.pojo.entity.disable.ChDisableApplyHis;
import com.hungraim.ltc.pojo.entity.disable.ChDisableInfo;
import com.hungraim.ltc.pojo.entity.serviceManage.ChCfgPara;
import com.hungraim.ltc.pojo.vo.assessment.ChAprTaskVo;
import com.hungraim.ltc.pojo.vo.assessment.ReTaskDistributionVO;
import com.hungraim.ltc.pojo.vo.disable.ChDisableApplyVo;
import com.hungraim.ltc.service.ReTaskService;
import com.hungraim.ltc.util.DateUtils;
import com.hungraim.ltc.util.Result;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * @author czz
 */
@Service
@Slf4j
@AllArgsConstructor
public class ReTaskServiceImpl extends ServiceImpl<TaskMapper, ChAprTask> implements ReTaskService {
    private final TaskMapper taskMapper;
    private final OrganFeignService organFeignService;
    private final ChDisableApplyMapper disableApplyMapper;
    private final ChDisableApplyMapper chDisableApplyMapper;
    private final ReTaskMapper reTaskMapper;
    private final DisableApplyHisMapper disableApplyHisMapper;
    private final CfgParaMapper cfgParaMapper;
    private final AprTaskMapper aprTaskMapper;
    private final DisableInfoMapper disableInfoMapper;
    private final DisableInfoHisMapper disableInfoHisMapper;

    @Override
    public Page<ReTaskDistributionVO> reTaskDistributionList(Page<ReTaskDistributionVO> page, Long organId, Boolean isIncluded, String acptProvincial, String acptCity, String acptCrea, String realName, String certiCode) {
        Result<List<Long>> organChild = null;
        List<Long> organIds = null;
        if (organId != null) {
            if (isIncluded == null) {
                isIncluded = false;
            }
            organChild = organFeignService.getOrganChild(organId, isIncluded);
            organIds = organChild.getData();
        }
        return taskMapper.reTaskDistributionList(page, organIds, acptProvincial, acptCity, acptCrea, realName, certiCode);
    }

    @Override
    public ChDisableApplyVo reassmentDetailById(String applyId) {
        ChDisableApplyVo chDisableApplyVo = null;
        chDisableApplyVo = chDisableApplyMapper.getReassmentDetailById(applyId);
        return chDisableApplyVo;
    }

    @Override
    public List<ChAprTaskVo> geReTaskHisInfoList(String disableInfoId) {
        // 历史评定信息列表
        return reTaskMapper.getReTaskHisInfoList(disableInfoId);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void disableExamine(String applyId, String checkName, Short applyStatus, String checkRemark) {
        Date date = new Date();
        //更新审核状态
        ChDisableApply chDisableApply = disableApplyMapper.selectById(applyId);
        chDisableApply.setCheckName(checkName);//审核人
        chDisableApply.setApplyStatus(applyStatus);//申请状态
        chDisableApply.setCheckRemark(checkRemark);//审核备注
        chDisableApply.setCheckTime(date);//审核时间
        disableApplyMapper.updateById(chDisableApply);
        //添加失能审核历史
        ChDisableApplyHis chDisableApplyHis = new ChDisableApplyHis();
        BeanUtils.copyProperties(chDisableApply, chDisableApplyHis);
        disableApplyHisMapper.insert(chDisableApplyHis);
        //审核通过
        if (applyStatus.equals(Short.valueOf("2"))) {
            //修改失能人员信息，首次申请只需新建
            List<ChDisableInfo> disableInfoList = disableInfoMapper.selectList(new LambdaQueryWrapper<ChDisableInfo>()
                    .eq(ChDisableInfo::getRealName, chDisableApply.getRealName())
                    .eq(ChDisableInfo::getCertiCode, chDisableApply.getCertiCode()));
            if (disableInfoList.size() != 0) {
                ChDisableInfo chDisableInfo = disableInfoList.get(0);
                // 先保存历史失能人员信息
                ChDisableInfoHis chDisableInfoHis = new ChDisableInfoHis();
                BeanUtil.copyProperties(chDisableInfo, chDisableInfoHis);
                chDisableInfoHis.setLcd(null);
                disableInfoHisMapper.insert(chDisableInfoHis);

                ChDisableInfo newDisableInfo = new ChDisableInfo();
                BeanUtil.copyProperties(chDisableApply, newDisableInfo);
                newDisableInfo.setDisabInfoId(chDisableInfo.getDisabInfoId());
                newDisableInfo.setDataFrom(0);//数据来源设置为0-申请
                disableInfoMapper.updateById(newDisableInfo);

                //添加失能评定信息
                ChAprTask chAprTask = new ChAprTask();
                ChCfgPara chCfgPara = cfgParaMapper.selectOne(new QueryWrapper<ChCfgPara>().eq("ORGAN_ID", chDisableApply.getOrganId()));
                if (chCfgPara != null && chCfgPara.getCfgArgTime() != null) {
                    Calendar cd = Calendar.getInstance();
                    cd.setTime(date);
                    cd.add(Calendar.DATE, chCfgPara.getCfgArgTime().intValue());
                    chAprTask.setLastAprDate(cd.getTime());//鉴定截至日期
                }
                BeanUtil.copyProperties(chDisableApply, chAprTask);
                chAprTask.setDisableInfoId(newDisableInfo.getDisabInfoId());//失能人员id
                chAprTask.setReAprStatus(0);
                chAprTask.setEffTime(new Date());
                chAprTask.setFcd(null);
                chAprTask.setAprType(1);//申请：0；复评：1
                chAprTask.setReAprStatus(1);
                chAprTask.setTaskStatus(0); // 任务状态 0 未分配
                aprTaskMapper.insert(chAprTask);
            }
        }
    }
}
