package com.hungraim.ltc.util;

import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.util.IOUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.UUID;

/**
 * 文件工具类
 */
@Slf4j
public class FileUtils {

    /**
     * 上传文件
     *
     * @param uploadFile controller接收到的文件对象
     * @param path       存放文件的根路径
     * @return 返回文件具体保存的路径，包含文件名称
     */
    public static String uploadFile(MultipartFile uploadFile, String path) {
        log.info("开始进行文件上传");
        try {
            //获取到文件名
            String fileName = uploadFile.getOriginalFilename();
            log.info("文件名称：{}", fileName);
            //使用日期格式创建文件夹
            String dataPathDir = new SimpleDateFormat("yyyy/MM/dd").format(new Date());
            //真实的文件夹路径
            String realDirPath = path + "/" + dataPathDir;
            //判断文件夹是否存在
            File dirFile = new File(realDirPath);
            if (!dirFile.exists()) {
                dirFile.mkdirs();
            }
            String uuid = UUID.randomUUID().toString().replace("-", "");
            //截取原先文件的类型
            String fileType = fileName.substring(fileName.lastIndexOf("."));
            String realName = uuid + fileType;
            //文件上传
            String realFilePath = realDirPath + "/" + realName;
            log.info("文件上传路径以及名称：{}", realFilePath);
            File file = new File(realFilePath);
            uploadFile.transferTo(file);
            return realFilePath;
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }


    /**
     *@author zwl
     *@desc 导出返回设置
     *@date 2023/1/12
     *@param  * @param workbook
     * @param fileName
     * @param response
     *@return void
     **/
    public static void  exportResponse(Workbook workbook,String fileName, HttpServletResponse response){
        try{
            response.setCharacterEncoding("UTF-8");
            response.setHeader("content-Type", "application/vnd.ms-excel");
            response.setHeader("Content-Disposition","attachment;filename=" + URLEncoder.encode(fileName,"UTF-8"));
            // 输出流
            workbook.write(response.getOutputStream());
            // 清除response
            response.flushBuffer();
        }catch (Exception e){
            log.error(e.getMessage());
        }
    }

    public static void downloadPathFile(String path,String fileName, HttpServletRequest request, HttpServletResponse response) {
        //设置文件路径
        File file = new File(path);
        //判断文件是否存在
        if (file.exists()) {
            FileInputStream fis = null;
            OutputStream os = null;
            try {
                // 取得输出流
                os = response.getOutputStream();
                // 清空输出流
                response.reset();
                //response.setContentType("application/force-download");// 设置强制下载不打开
                response.setCharacterEncoding("UTF-8");
                response.setHeader("content-Type", "application/vnd.ms-excel");
                response.setHeader("Content-Disposition","attachment;filename=" + URLEncoder.encode(fileName,"UTF-8"));
                fis = new FileInputStream(file);
                IOUtils.copy(fis, os);
                response.getOutputStream().flush();
              /*  bis = new BufferedInputStream(fis);
                OutputStream os = response.getOutputStream();
                int i = bis.read(buffer);
                while (i != -1) {
                    os.write(buffer, 0, i);
                    i = bis.read(buffer);
                }*/
                log.info("=========下载成功:{}",fileName);
            } catch (Exception e) {
              log.error("=====error1:{}",e);
            } finally {
                try {
                    if (fis != null) {
                        fis.close();
                    }
                } catch (IOException e) {
                   // logger.error(ExceptionUtils.getFullStackTrace(e));
                }
                try {
                    if (os != null) {
                        os.close();
                    }
                } catch (IOException e) {
                    //logger.error(ExceptionUtils.getFullStackTrace(e));
                }
            }
        }
    }

    /**
     * 文件下载
     *
     * @param fileName
     * @return
     * @author 小辰哥哥
     */

    public static byte[] downloadFile(String path,String fileName, HttpServletResponse response) {
        // 关联文件
        File file = new File(path);
        byte[] fileBytes = null;
        FileInputStream fileInputStream = null;
        ByteArrayOutputStream bos = null;
//设置响应头类型，这里可以根据文件类型设置，text/plain、application/vnd.ms-excel等
        response.setHeader("content-type", "text/plain;charset=UTF-8");
        response.setContentType("application/octet-stream;charset=UTF-8");
        //如果不设置响应头大小，可能报错：“Excel 已完成文件级验证和修复。此工作簿的某些部分可能已被修复或丢弃”
        response.addHeader("Content-Length", String.valueOf(file.length()));
        try {
            response.setHeader("Content-Disposition","attachment;filename=" + URLEncoder.encode(fileName,"UTF-8"));

            fileInputStream = new FileInputStream(file);
            bos = new ByteArrayOutputStream();
            byte[] bytes = new byte[1024];
            int len ;
            while ((len = fileInputStream.read(bytes)) != -1) {
                bos.write(bytes, 0, len);
            }
            fileBytes = bos.toByteArray();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                fileInputStream.close();
                bos.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return fileBytes;
    }
}

