package com.hungraim.ltc.governance.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hungraim.ltc.api.AttachFeignService;
import com.hungraim.ltc.governance.config.PicRoleConfig;
import com.hungraim.ltc.governance.dao.BuildBedMapper;
import com.hungraim.ltc.governance.dao.ServiceTypeDetailedMapper;
import com.hungraim.ltc.governance.dao.SrvSendMapper;
import com.hungraim.ltc.governance.service.IBuildBedService;
import com.hungraim.ltc.pojo.entity.SysConfig.ChFndAttach;
import com.hungraim.ltc.pojo.entity.account.ChPunchDetailVO;
import com.hungraim.ltc.pojo.entity.account.ChPunchDetailsVO;
import com.hungraim.ltc.pojo.entity.account.ChServiceTypeDetailedVO;
import com.hungraim.ltc.pojo.entity.account.ChServiceTypeVO;
import com.hungraim.ltc.pojo.entity.governance.ChSrvBuildBed;
import com.hungraim.ltc.pojo.vo.assessment.TaskReportVO;
import com.hungraim.ltc.pojo.vo.disable.ChSrvTaskDetailVo;
import com.hungraim.ltc.pojo.vo.disable.PlaceOtherVO;
import com.hungraim.ltc.pojo.vo.governance.BuildBedDetailVo;
import com.hungraim.ltc.pojo.vo.governance.BuildBedSearchReq;
import com.hungraim.ltc.pojo.vo.system.AttachUpdateVo;
import com.hungraim.ltc.util.AttachType;
import com.hungraim.ltc.util.Result;
import com.hungraim.ltc.util.ResultCode;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author jiaguokai
 */
@Slf4j
@Service
@AllArgsConstructor
public class BuildBedServiceImpl extends ServiceImpl<BuildBedMapper, ChSrvBuildBed> implements IBuildBedService {
    private final BuildBedMapper buildBedMapper;
    private final AttachFeignService attachFeignService;

    private final ServiceTypeDetailedMapper serviceTypeDetailed;

    @Autowired
    private PicRoleConfig picRoleConfig;

    @Resource
    private SrvSendMapper srvSendMapper;

    @Override
    public Result searchOrgBed(BuildBedSearchReq req) {
        if (2 == req.getStatus()) {
            req.setStatus(null);
        }
        Page page = buildBedMapper.searchOrgBed(new Page<>(req.getPageNum(), req.getPageSize()), req);
        return Result.success(page);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveOrUpdateBuildBed(BuildBedDetailVo buildBedDetailVo) {
        ChSrvBuildBed chSrvBuildBed = new ChSrvBuildBed();
        BeanUtils.copyProperties(buildBedDetailVo, chSrvBuildBed);
        chSrvBuildBed.setStatus(0); // 状态（0已建床，1已撤床）
        chSrvBuildBed.setAccountStatus(0); // 结算状态（0，未结算；1，结算中；2，已结算）
        this.saveOrUpdate(chSrvBuildBed);
        buildBedDetailVo.setBuildBedId(chSrvBuildBed.getBuildBedId());
        this.updateAttachs(buildBedDetailVo);
    }

    @Override
    public BuildBedDetailVo getBuildBedDetail(Integer buildBedId) {
        ChSrvBuildBed chSrvBuildBed = this.getById(buildBedId);
        BuildBedDetailVo buildBedDetailVo = new BuildBedDetailVo();
        BeanUtils.copyProperties(chSrvBuildBed, buildBedDetailVo);
        try {
            Result<List<ChFndAttach>> result = attachFeignService.selectAttachs(buildBedId.longValue(), AttachType.BUILD_BED.getUploadType());
            if (ResultCode.SUCCESS.getCode().equals(result.getCode())) {
                buildBedDetailVo.setFileDataList(result.getData());
            } else {
                log.error("请求获取建床附件失败：{}", result.getMsg());
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("请求获取建床附件失败：{}", e.getMessage());
        }
        return buildBedDetailVo;
    }

    /**
     * 打卡查询
     * @param page
     * @param
     * @return
     */
    @Override
    public Page<ChSrvTaskDetailVo> chSrvTaskDetailQuery(Page<ChSrvTaskDetailVo> page,ChSrvTaskDetailVo vo){

        Page<ChSrvTaskDetailVo> chSrvTaskDetailVoPage = buildBedMapper.chSrvTaskDetailQuery(page, vo);
        chSrvTaskDetailVoPage.getRecords().stream().filter(chSrvTaskDetail -> {
            if (null != chSrvTaskDetail.getSevStarTime()){
                chSrvTaskDetail.setSevStarTime(chSrvTaskDetail.getSevStarTime().substring(0,19));
            }
            if (null != chSrvTaskDetail.getSevEndTime()){
                chSrvTaskDetail.setSevEndTime(chSrvTaskDetail.getSevEndTime().substring(0,19));
            }
            return true;
        }).collect(Collectors.toList());
        return chSrvTaskDetailVoPage;
    }
    /**
     * 删除打卡数据
     * @param taskDetailId
     * @param userName
     * @return
     */
    @Override
    public int chSrvTaskDetailDelete(Long taskDetailId, String userName) {
        return buildBedMapper.deleteById(taskDetailId);
    }

    /**
     * 打卡详情
     * @param taskDetailId
     */
    @Override
    public ChPunchDetailsVO chPunchDetailsQuery(Long taskDetailId,String certiCode, String sevStarTime,String sevEndTime,Integer userId,Integer srvOrganId) {
        List<ChPunchDetailVO> chPunchDetailVOS = new ArrayList<>();
        //获取项目信息
        List<ChServiceTypeVO> chServiceTypeDetaileds = buildBedMapper.selectServiceTypeDetailed();
        log.info("chPunchDetailsQuery项目信息:"+chServiceTypeDetaileds);
        Map<String, List<ChServiceTypeVO>> collects = chServiceTypeDetaileds.stream().collect(Collectors.groupingBy(ChServiceTypeVO::getServiceTypeName));

        Iterator<Map.Entry<String, List<ChServiceTypeVO>>> iterator = collects.entrySet().iterator();
        while (iterator.hasNext()){
            Map.Entry<String, List<ChServiceTypeVO>> next = iterator.next();
            ChPunchDetailVO chPunchDetailVO=new ChPunchDetailVO();
            chPunchDetailVO.setServiceTypeName(next.getKey());
            chPunchDetailVO.setServiceTypeVOS(next.getValue());
            chPunchDetailVOS.add(chPunchDetailVO);
        }
        //获取服务机构和服务类型
        ChPunchDetailsVO chPunchDetailsVO = buildBedMapper.getsrvOrgan(taskDetailId);
        if(chPunchDetailsVO==null){
            chPunchDetailsVO=new ChPunchDetailsVO();
        }
        String month=sevStarTime.substring(5,7);//获取当前打卡月份
        if(month.startsWith("0")){
            month = month.replaceFirst("0","");
        }
        chPunchDetailsVO.setMonth(month+"月");//返回月份
        log.info("chPunchDetailsQuery获取服务机构和服务类型:"+chPunchDetailsVO);
        //打卡详情
        String modeType = buildBedMapper.selectModeTypeById(taskDetailId);
        List<ChServiceTypeDetailedVO> serviceTypeDetailed1s = new ArrayList<ChServiceTypeDetailedVO>();
        int sendCount=srvSendMapper.getCountSrvSend(certiCode);
        if(sendCount>1) {//如果有多个分配记录，则按该方法查询打卡列表
            serviceTypeDetailed1s = buildBedMapper.getServiceTypeDetailedBySrvPlan(taskDetailId,certiCode,sevStarTime, sevEndTime,modeType,srvOrganId);
        }else{//否则还按原先逻辑
            serviceTypeDetailed1s = buildBedMapper.getServiceTypeDetailed(taskDetailId,certiCode,sevStarTime, sevEndTime,modeType,srvOrganId);
        }

        List<String> roleIds = picRoleConfig.getRoleId();
        log.info("==========chPunchDetailsQuery====roleIds:{}",roleIds);
        int roleByUserId = buildBedMapper.getRoleByUserId(userId,roleIds);
        for (ChServiceTypeDetailedVO serviceTypeDetailed1 : serviceTypeDetailed1s) {
            String srvStartPic = serviceTypeDetailed1.getSrvStartPic();
            String srvStartPic2 = serviceTypeDetailed1.getSrvStartPic2();
            String srvEndPic = serviceTypeDetailed1.getSrvEndPic();
            List<String> srvPicList =   new ArrayList<>();
            if(srvStartPic!=null) {
                srvPicList.add(srvStartPic);
            }
            // 机构端能护理人员的拍照和第三张拍照，其他都能看到 等于0角色不是机构端
        //    if(roleByUserId == 0 && srvStartPic2!=null) {
                srvPicList.add(srvStartPic2);//2024年8月29，和用户确认第二张照片开放
          //  }
            if(srvEndPic!=null) {
                srvPicList.add(srvEndPic);
            }
            serviceTypeDetailed1.setSrvPicList(srvPicList);
            chPunchDetailsVO.setRealName(serviceTypeDetailed1.getRealName());//失能人员姓名
        }
        log.info("chPunchDetailsQuery打卡详情:"+serviceTypeDetailed1s);
        //获取打卡列表底下总时长
        List<Map> timesMap=buildBedMapper.getSrvSumTime(taskDetailId,certiCode,sevStarTime, sevEndTime,modeType);
        log.info("timesMap打卡时长:"+timesMap);
        chPunchDetailsVO.setRecordData(chPunchDetailVOS);
        chPunchDetailsVO.setChPunchDetailVOS(serviceTypeDetailed1s);
        chPunchDetailsVO.setSrvSumTimeVOS(timesMap);//打卡列表底下总时长
        log.info("chPunchDetailsQuery:"+chPunchDetailsVO);
        return chPunchDetailsVO;
    }

    /**
     * 调用feign api来修改附件数据
     *
     * @param buildBedDetailVo 建床详情封装对象
     */
    private void updateAttachs(BuildBedDetailVo buildBedDetailVo) {
        // zhangch 20210525 新增附件修改逻辑
        List<ChFndAttach> fileDataList = buildBedDetailVo.getFileDataList();
        AttachUpdateVo attachUpdateVo = new AttachUpdateVo();
        attachUpdateVo.setCtrlId(buildBedDetailVo.getBuildBedId().longValue());
        attachUpdateVo.setAttachType(AttachType.BUILD_BED.getUploadType());
        attachUpdateVo.setChFndAttachs(fileDataList);
        attachFeignService.updateAttachs(attachUpdateVo);
    }


}
