package com.hungraim.ltc.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.hungraim.ltc.pojo.entity.SysConfig.ChFndAttach;
import com.hungraim.ltc.pojo.vo.system.AttachUpdateVo;
import com.hungraim.ltc.system.dao.ChFndAttachMapper;
import com.hungraim.ltc.system.service.AttachService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Objects;

@Service
public class AttachServiceImpl implements AttachService {

    @Autowired
    private ChFndAttachMapper chFndAttachMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long saveFileInfo(String originalFilename, String realPath, Short attachType) {
        ChFndAttach chFndAttach = new ChFndAttach();
        chFndAttach.setFileName(originalFilename);
        chFndAttach.setFilePath(realPath);
        chFndAttach.setFileTime(new Date());
        chFndAttach.setAttachType(attachType);
        chFndAttach.setStatus((short) 0);
        chFndAttachMapper.insert(chFndAttach);
        return chFndAttach.getAttachId();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateAttachs(AttachUpdateVo attachUpdateVo) {
        Long ctrlId = attachUpdateVo.getCtrlId();
        Short attachType = attachUpdateVo.getAttachType();
        List<ChFndAttach> chFndAttachs = attachUpdateVo.getChFndAttachs();
        if (chFndAttachs != null) {
            for (ChFndAttach chFndAttach : chFndAttachs) {
                chFndAttach.setCtrlId(ctrlId);
                chFndAttach.setAttachType(attachType);
            }
        }
        List<ChFndAttach> dbChFndAttachs = this.getChFndAttaches(ctrlId, attachType);
        if (dbChFndAttachs != null && dbChFndAttachs.size() > 0) {
            /**
             *   如果查询到了对应该ctrlId对应的数据，就是更新ctrlId对应的数据集.分2步
             *   1->先把原有数据列表中在新数据列表中找不到的数据改为失效状态
             *   2->把新数据列表中在原有数据中找不到的数据的增加ctrlId字段的修改
             */
            // step 1
            for (ChFndAttach dbChFndAttach : dbChFndAttachs) {
                boolean deleteFlag = true;
                if (chFndAttachs != null) {
                    for (ChFndAttach chfndAttach : chFndAttachs) {
                        if (dbChFndAttach.getAttachId().equals(chfndAttach.getAttachId())) {
                            deleteFlag = false;
                        }
                    }
                }
                if (deleteFlag) {
                    dbChFndAttach.setStatus((short) 1);
                    chFndAttachMapper.updateById(dbChFndAttach);
                }
            }
            // step 2
            if (chFndAttachs != null) {
                for (ChFndAttach chFndAttach : chFndAttachs) {
                    boolean updateFlag = true;
                    for (ChFndAttach dbChFndAttach : dbChFndAttachs) {
                        if (chFndAttach.getAttachId().equals(dbChFndAttach.getAttachId())) {
                            updateFlag = false;
                        }
                    }
                    if (updateFlag) {
                        chFndAttachMapper.updateById(chFndAttach);
                    }
                }
            }
        } else {
            if (chFndAttachs != null) {
                // 如果没有查询到该ctrlId对应的数据，就是直接更新对应attachId的ctrlId字段
                for (ChFndAttach chFndAttach : chFndAttachs) {
                    chFndAttachMapper.updateById(chFndAttach);
                }
            }
        }
    }

    private List<ChFndAttach> getChFndAttaches(Long ctrlId, Short attachType) {
        // 查询出数据库中原有的该ctrlId关联的数据
        QueryWrapper<ChFndAttach> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("CTRL_ID", ctrlId);
        queryWrapper.eq("ATTACH_TYPE", attachType);
        queryWrapper.eq("STATUS", (short) 0);
        List<ChFndAttach> dbChFndAttachs = chFndAttachMapper.selectList(queryWrapper);
        return dbChFndAttachs;
    }

    @Override
    public List<ChFndAttach> selectAttachs(Long ctrlId, Short attachType) {
        return this.getChFndAttaches(ctrlId, attachType);
    }


    @Override
    public ChFndAttach selectById(Long attachId){
        ChFndAttach chFndAttach = chFndAttachMapper.selectById(attachId);
        return chFndAttach;
    }
}
